"""Raw representations of every data type in the AWS Lambda service.

See Also:
    `AWS developer guide for Lambda
    <https://docs.aws.amazon.com/lambda/latest/dg/welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from typing import Any
from typing import Dict

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Alias",
    "AliasProperties",
    "EventSourceMapping",
    "EventSourceMappingProperties",
    "Function",
    "FunctionProperties",
    "LayerVersion",
    "LayerVersionProperties",
    "LayerVersionPermission",
    "LayerVersionPermissionProperties",
    "Permission",
    "PermissionProperties",
    "Version",
    "VersionProperties",
]


@attrs(**ATTRSCONFIG)
class AliasProperties(ResourceProperties):
    Description = attrib(default=None)
    FunctionName = attrib(default=None)
    FunctionVersion = attrib(default=None)
    Name = attrib(default=None)
    RoutingConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Alias(Resource):
    """A Alias for Lambda.

    See Also:
        `AWS Cloud Formation documentation for Alias
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::Alias"

    Properties: AliasProperties = attrib(
        factory=AliasProperties,
        converter=create_object_converter(AliasProperties),
    )

    # NB: UpdatePolicy may be set for Alias
    # (unlike most Resource types)
    UpdatePolicy: Dict[str, Any] = attrib(factory=dict)


@attrs(**ATTRSCONFIG)
class EventSourceMappingProperties(ResourceProperties):
    BatchSize = attrib(default=None)
    Enabled = attrib(default=None)
    EventSourceArn = attrib(default=None)
    FunctionName = attrib(default=None)
    StartingPosition = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EventSourceMapping(Resource):
    """A Event Source Mapping for Lambda.

    See Also:
        `AWS Cloud Formation documentation for EventSourceMapping
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::EventSourceMapping"

    Properties: EventSourceMappingProperties = attrib(
        factory=EventSourceMappingProperties,
        converter=create_object_converter(EventSourceMappingProperties),
    )


@attrs(**ATTRSCONFIG)
class FunctionProperties(ResourceProperties):
    Code = attrib(default=None)
    DeadLetterConfig = attrib(default=None)
    Description = attrib(default=None)
    Environment = attrib(default=None)
    FunctionName = attrib(default=None)
    Handler = attrib(default=None)
    KmsKeyArn = attrib(default=None)
    Layers = attrib(default=None)
    MemorySize = attrib(default=None)
    ReservedConcurrentExecutions = attrib(default=None)
    Role = attrib(default=None)
    Runtime = attrib(default=None)
    Tags = attrib(default=None)
    Timeout = attrib(default=None)
    TracingConfig = attrib(default=None)
    VpcConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Function(Resource):
    """A Function for Lambda.

    See Also:
        `AWS Cloud Formation documentation for Function
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::Function"

    Properties: FunctionProperties = attrib(
        factory=FunctionProperties,
        converter=create_object_converter(FunctionProperties),
    )


@attrs(**ATTRSCONFIG)
class LayerVersionProperties(ResourceProperties):
    CompatibleRuntimes = attrib(default=None)
    Content = attrib(default=None)
    Description = attrib(default=None)
    LayerName = attrib(default=None)
    LicenseInfo = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LayerVersion(Resource):
    """A Layer Version for Lambda.

    See Also:
        `AWS Cloud Formation documentation for LayerVersion
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-layerversion.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::LayerVersion"

    Properties: LayerVersionProperties = attrib(
        factory=LayerVersionProperties,
        converter=create_object_converter(LayerVersionProperties),
    )


@attrs(**ATTRSCONFIG)
class LayerVersionPermissionProperties(ResourceProperties):
    Action = attrib(default=None)
    LayerVersionArn = attrib(default=None)
    OrganizationId = attrib(default=None)
    Principal = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LayerVersionPermission(Resource):
    """A Layer Version Permission for Lambda.

    See Also:
        `AWS Cloud Formation documentation for LayerVersionPermission
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-layerversionpermission.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::LayerVersionPermission"

    Properties: LayerVersionPermissionProperties = attrib(
        factory=LayerVersionPermissionProperties,
        converter=create_object_converter(LayerVersionPermissionProperties),
    )


@attrs(**ATTRSCONFIG)
class PermissionProperties(ResourceProperties):
    Action = attrib(default=None)
    EventSourceToken = attrib(default=None)
    FunctionName = attrib(default=None)
    Principal = attrib(default=None)
    SourceAccount = attrib(default=None)
    SourceArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Permission(Resource):
    """A Permission for Lambda.

    See Also:
        `AWS Cloud Formation documentation for Permission
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::Permission"

    Properties: PermissionProperties = attrib(
        factory=PermissionProperties,
        converter=create_object_converter(PermissionProperties),
    )


@attrs(**ATTRSCONFIG)
class VersionProperties(ResourceProperties):
    CodeSha256 = attrib(default=None)
    Description = attrib(default=None)
    FunctionName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Version(Resource):
    """A Version for Lambda.

    See Also:
        `AWS Cloud Formation documentation for Version
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html>`_
    """

    RESOURCE_TYPE = "AWS::Lambda::Version"

    Properties: VersionProperties = attrib(
        factory=VersionProperties,
        converter=create_object_converter(VersionProperties),
    )



"""General-use classes to interact with the Glue service through CloudFormation.

See Also:
    `AWS developer guide for Glue
    <https://docs.aws.amazon.com/glue/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import glue as _raw

# noinspection PyUnresolvedReferences
from .._raw.glue import *



"""General-use classes to interact with the ApiGateway service through CloudFormation.

See Also:
    `AWS developer guide for ApiGateway
    <https://docs.aws.amazon.com/apigateway/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import apigateway as _raw

# noinspection PyUnresolvedReferences
from .._raw.apigateway import *



"""Raw representations of every data type in the AWS ElasticBeanstalk service.

See Also:
    `AWS developer guide for ElasticBeanstalk
    <https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Application",
    "ApplicationProperties",
    "ApplicationVersion",
    "ApplicationVersionProperties",
    "ConfigurationTemplate",
    "ConfigurationTemplateProperties",
    "Environment",
    "EnvironmentProperties",
]


@attrs(**ATTRSCONFIG)
class ApplicationProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    Description = attrib(default=None)
    ResourceLifecycleConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Application(Resource):
    """A Application for ElasticBeanstalk.

    See Also:
        `AWS Cloud Formation documentation for Application
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticBeanstalk::Application"

    Properties: ApplicationProperties = attrib(
        factory=ApplicationProperties,
        converter=create_object_converter(ApplicationProperties),
    )


@attrs(**ATTRSCONFIG)
class ApplicationVersionProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    Description = attrib(default=None)
    SourceBundle = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ApplicationVersion(Resource):
    """A Application Version for ElasticBeanstalk.

    See Also:
        `AWS Cloud Formation documentation for ApplicationVersion
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticBeanstalk::ApplicationVersion"

    Properties: ApplicationVersionProperties = attrib(
        factory=ApplicationVersionProperties,
        converter=create_object_converter(ApplicationVersionProperties),
    )


@attrs(**ATTRSCONFIG)
class ConfigurationTemplateProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    Description = attrib(default=None)
    EnvironmentId = attrib(default=None)
    OptionSettings = attrib(default=None)
    PlatformArn = attrib(default=None)
    SolutionStackName = attrib(default=None)
    SourceConfiguration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ConfigurationTemplate(Resource):
    """A Configuration Template for ElasticBeanstalk.

    See Also:
        `AWS Cloud Formation documentation for ConfigurationTemplate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticbeanstalk-configurationtemplate.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticBeanstalk::ConfigurationTemplate"

    Properties: ConfigurationTemplateProperties = attrib(
        factory=ConfigurationTemplateProperties,
        converter=create_object_converter(ConfigurationTemplateProperties),
    )


@attrs(**ATTRSCONFIG)
class EnvironmentProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    CNAMEPrefix = attrib(default=None)
    Description = attrib(default=None)
    EnvironmentName = attrib(default=None)
    OptionSettings = attrib(default=None)
    PlatformArn = attrib(default=None)
    SolutionStackName = attrib(default=None)
    Tags = attrib(default=None)
    TemplateName = attrib(default=None)
    Tier = attrib(default=None)
    VersionLabel = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Environment(Resource):
    """A Environment for ElasticBeanstalk.

    See Also:
        `AWS Cloud Formation documentation for Environment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticBeanstalk::Environment"

    Properties: EnvironmentProperties = attrib(
        factory=EnvironmentProperties,
        converter=create_object_converter(EnvironmentProperties),
    )



"""Raw representations of every data type in the AWS RoboMaker service.

See Also:
    `AWS developer guide for RoboMaker
    <https://docs.aws.amazon.com/robomaker/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Fleet",
    "FleetProperties",
    "Robot",
    "RobotProperties",
    "RobotApplication",
    "RobotApplicationProperties",
    "RobotApplicationVersion",
    "RobotApplicationVersionProperties",
    "SimulationApplication",
    "SimulationApplicationProperties",
    "SimulationApplicationVersion",
    "SimulationApplicationVersionProperties",
]


@attrs(**ATTRSCONFIG)
class FleetProperties(ResourceProperties):
    Name = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Fleet(Resource):
    """A Fleet for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for Fleet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-fleet.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::Fleet"

    Properties: FleetProperties = attrib(
        factory=FleetProperties,
        converter=create_object_converter(FleetProperties),
    )


@attrs(**ATTRSCONFIG)
class RobotProperties(ResourceProperties):
    Architecture = attrib(default=None)
    Fleet = attrib(default=None)
    GreengrassGroupId = attrib(default=None)
    Name = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Robot(Resource):
    """A Robot for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for Robot
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-robot.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::Robot"

    Properties: RobotProperties = attrib(
        factory=RobotProperties,
        converter=create_object_converter(RobotProperties),
    )


@attrs(**ATTRSCONFIG)
class RobotApplicationProperties(ResourceProperties):
    CurrentRevisionId = attrib(default=None)
    Name = attrib(default=None)
    RobotSoftwareSuite = attrib(default=None)
    Sources = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RobotApplication(Resource):
    """A Robot Application for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for RobotApplication
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-robotapplication.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::RobotApplication"

    Properties: RobotApplicationProperties = attrib(
        factory=RobotApplicationProperties,
        converter=create_object_converter(RobotApplicationProperties),
    )


@attrs(**ATTRSCONFIG)
class RobotApplicationVersionProperties(ResourceProperties):
    Application = attrib(default=None)
    CurrentRevisionId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RobotApplicationVersion(Resource):
    """A Robot Application Version for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for RobotApplicationVersion
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-robotapplicationversion.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::RobotApplicationVersion"

    Properties: RobotApplicationVersionProperties = attrib(
        factory=RobotApplicationVersionProperties,
        converter=create_object_converter(RobotApplicationVersionProperties),
    )


@attrs(**ATTRSCONFIG)
class SimulationApplicationProperties(ResourceProperties):
    CurrentRevisionId = attrib(default=None)
    Name = attrib(default=None)
    RenderingEngine = attrib(default=None)
    RobotSoftwareSuite = attrib(default=None)
    SimulationSoftwareSuite = attrib(default=None)
    Sources = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SimulationApplication(Resource):
    """A Simulation Application for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for SimulationApplication
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-simulationapplication.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::SimulationApplication"

    Properties: SimulationApplicationProperties = attrib(
        factory=SimulationApplicationProperties,
        converter=create_object_converter(SimulationApplicationProperties),
    )


@attrs(**ATTRSCONFIG)
class SimulationApplicationVersionProperties(ResourceProperties):
    Application = attrib(default=None)
    CurrentRevisionId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class SimulationApplicationVersion(Resource):
    """A Simulation Application Version for RoboMaker.

    See Also:
        `AWS Cloud Formation documentation for SimulationApplicationVersion
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-robomaker-simulationapplicationversion.html>`_
    """

    RESOURCE_TYPE = "AWS::RoboMaker::SimulationApplicationVersion"

    Properties: SimulationApplicationVersionProperties = attrib(
        factory=SimulationApplicationVersionProperties,
        converter=create_object_converter(SimulationApplicationVersionProperties),
    )



"""Raw representations of every data type in the AWS OpsWorks service.

See Also:
    `AWS developer guide for OpsWorks
    <https://docs.aws.amazon.com/opsworks/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "App",
    "AppProperties",
    "ElasticLoadBalancerAttachment",
    "ElasticLoadBalancerAttachmentProperties",
    "Instance",
    "InstanceProperties",
    "Layer",
    "LayerProperties",
    "Stack",
    "StackProperties",
    "UserProfile",
    "UserProfileProperties",
    "Volume",
    "VolumeProperties",
]


@attrs(**ATTRSCONFIG)
class AppProperties(ResourceProperties):
    AppSource = attrib(default=None)
    Attributes = attrib(default=None)
    DataSources = attrib(default=None)
    Description = attrib(default=None)
    Domains = attrib(default=None)
    EnableSsl = attrib(default=None)
    Environment = attrib(default=None)
    Name = attrib(default=None)
    Shortname = attrib(default=None)
    SslConfiguration = attrib(default=None)
    StackId = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class App(Resource):
    """A App for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for App
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::App"

    Properties: AppProperties = attrib(
        factory=AppProperties,
        converter=create_object_converter(AppProperties),
    )


@attrs(**ATTRSCONFIG)
class ElasticLoadBalancerAttachmentProperties(ResourceProperties):
    ElasticLoadBalancerName = attrib(default=None)
    LayerId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ElasticLoadBalancerAttachment(Resource):
    """A Elastic Load Balancer Attachment for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for ElasticLoadBalancerAttachment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::ElasticLoadBalancerAttachment"

    Properties: ElasticLoadBalancerAttachmentProperties = attrib(
        factory=ElasticLoadBalancerAttachmentProperties,
        converter=create_object_converter(ElasticLoadBalancerAttachmentProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceProperties(ResourceProperties):
    AgentVersion = attrib(default=None)
    AmiId = attrib(default=None)
    Architecture = attrib(default=None)
    AutoScalingType = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    BlockDeviceMappings = attrib(default=None)
    EbsOptimized = attrib(default=None)
    ElasticIps = attrib(default=None)
    Hostname = attrib(default=None)
    InstallUpdatesOnBoot = attrib(default=None)
    InstanceType = attrib(default=None)
    LayerIds = attrib(default=None)
    Os = attrib(default=None)
    RootDeviceType = attrib(default=None)
    SshKeyName = attrib(default=None)
    StackId = attrib(default=None)
    SubnetId = attrib(default=None)
    Tenancy = attrib(default=None)
    TimeBasedAutoScaling = attrib(default=None)
    VirtualizationType = attrib(default=None)
    Volumes = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Instance(Resource):
    """A Instance for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for Instance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::Instance"

    Properties: InstanceProperties = attrib(
        factory=InstanceProperties,
        converter=create_object_converter(InstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class LayerProperties(ResourceProperties):
    Attributes = attrib(default=None)
    AutoAssignElasticIps = attrib(default=None)
    AutoAssignPublicIps = attrib(default=None)
    CustomInstanceProfileArn = attrib(default=None)
    CustomJson = attrib(default=None)
    CustomRecipes = attrib(default=None)
    CustomSecurityGroupIds = attrib(default=None)
    EnableAutoHealing = attrib(default=None)
    InstallUpdatesOnBoot = attrib(default=None)
    LifecycleEventConfiguration = attrib(default=None)
    LoadBasedAutoScaling = attrib(default=None)
    Name = attrib(default=None)
    Packages = attrib(default=None)
    Shortname = attrib(default=None)
    StackId = attrib(default=None)
    Tags = attrib(default=None)
    Type = attrib(default=None)
    UseEbsOptimizedInstances = attrib(default=None)
    VolumeConfigurations = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Layer(Resource):
    """A Layer for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for Layer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::Layer"

    Properties: LayerProperties = attrib(
        factory=LayerProperties,
        converter=create_object_converter(LayerProperties),
    )


@attrs(**ATTRSCONFIG)
class StackProperties(ResourceProperties):
    AgentVersion = attrib(default=None)
    Attributes = attrib(default=None)
    ChefConfiguration = attrib(default=None)
    CloneAppIds = attrib(default=None)
    ClonePermissions = attrib(default=None)
    ConfigurationManager = attrib(default=None)
    CustomCookbooksSource = attrib(default=None)
    CustomJson = attrib(default=None)
    DefaultAvailabilityZone = attrib(default=None)
    DefaultInstanceProfileArn = attrib(default=None)
    DefaultOs = attrib(default=None)
    DefaultRootDeviceType = attrib(default=None)
    DefaultSshKeyName = attrib(default=None)
    DefaultSubnetId = attrib(default=None)
    EcsClusterArn = attrib(default=None)
    ElasticIps = attrib(default=None)
    HostnameTheme = attrib(default=None)
    Name = attrib(default=None)
    RdsDbInstances = attrib(default=None)
    ServiceRoleArn = attrib(default=None)
    SourceStackId = attrib(default=None)
    Tags = attrib(default=None)
    UseCustomCookbooks = attrib(default=None)
    UseOpsworksSecurityGroups = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stack(Resource):
    """A Stack for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for Stack
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::Stack"

    Properties: StackProperties = attrib(
        factory=StackProperties,
        converter=create_object_converter(StackProperties),
    )


@attrs(**ATTRSCONFIG)
class UserProfileProperties(ResourceProperties):
    AllowSelfManagement = attrib(default=None)
    IamUserArn = attrib(default=None)
    SshPublicKey = attrib(default=None)
    SshUsername = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserProfile(Resource):
    """A User Profile for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for UserProfile
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::UserProfile"

    Properties: UserProfileProperties = attrib(
        factory=UserProfileProperties,
        converter=create_object_converter(UserProfileProperties),
    )


@attrs(**ATTRSCONFIG)
class VolumeProperties(ResourceProperties):
    Ec2VolumeId = attrib(default=None)
    MountPoint = attrib(default=None)
    Name = attrib(default=None)
    StackId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Volume(Resource):
    """A Volume for OpsWorks.

    See Also:
        `AWS Cloud Formation documentation for Volume
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html>`_
    """

    RESOURCE_TYPE = "AWS::OpsWorks::Volume"

    Properties: VolumeProperties = attrib(
        factory=VolumeProperties,
        converter=create_object_converter(VolumeProperties),
    )



"""General-use classes to interact with the StepFunctions service through CloudFormation.

See Also:
    `AWS developer guide for StepFunctions
    <https://docs.aws.amazon.com/step-functions/latest/dg/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import stepfunctions as _raw

# noinspection PyUnresolvedReferences
from .._raw.stepfunctions import *



"""Represent the application of CloudFormation intrinsic functions.

These will probably return a special object that exports as a YAML scalar
("string") containing the function reference. Some functions may be able to
be resolved internally.

See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference.html
"""

from typing import Union

import yaml

from flyingcircus.core import AWS_Region
from flyingcircus.core import PseudoParameter
from .yaml import CustomYamlObject
from .yaml import represent_string


# TODO use the rule that every time you detect a nested func, the outer has to use long form

class _Function(CustomYamlObject):
    """Base class for all CloudFormation intrinsic functions"""

    def _get_string_node(self, dumper: yaml.Dumper, value: Union["_Function", str], tag: str) -> yaml.Node:
        """Get a PyYAML node for a function that returns a string.

        Lots of the intrinsic functions take a single string as the argument,
        which we echo directly into the output. However, in CloudFormation,
        this string could be a reference to another function rather than a
        literal string, and that needs to be handled specially.
        """
        if isinstance(value, _Function):
            return dumper.represent_dict({
                f"Fn::{tag}": value
            })
        else:
            # TODO be able to control the scalar output, perhaps
            # TODO use represent_string instead, perhaps
            return dumper.represent_scalar(f"!{tag}", value, style="")


class Base64(_Function):
    """Models the behaviour of Fn::Base64 for Python objects.

    Will have the effect of turning the content string into a base64-encoded
    string when the Cloud Formation template is executed.

    See https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-base64.html
    """

    def __init__(self, data):
        self._data = data

    def as_yaml_node(self, dumper):
        # TODO If the content is a string, try to format multi-line strings nicely (eg. from EC2 UserData)
        return self._get_string_node(dumper, self._data, "Base64")


class GetAtt(_Function):
    """Models the behaviour of Fn::GetAtt for Python objects.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-getavailabilityzones.html
    """

    # TODO Verify that attribute name is valid by comparing it against known
    #  attributes for the resource type in question. This is obviously only
    #  possible if the resource is concrete Resource subclass, and the
    #  attribute name is composed solely of strings.

    # TODO Have AWS attribute access as a special function on the resource
    #  and/or be actual attribute lookup (the latter is perhaps too confusing)

    def __init__(self, resource, *attribute_name):
        """Get a resource attribute from another resource in this stack.

        :param resource: The Python object that contains the resource attribute.
        :param attribute_name: One or more components for the name of the attribute.
            The components may be supplied as a list, or as a single dotted
            string. They may contain Ref objects.
        """
        if isinstance(resource, str):
            raise TypeError("You can't directly create a GetAtt on a logical name.")

        if len(attribute_name) < 1:
            raise ValueError("At least one part of the AWS attribute name is required")

        self._attribute_name_has_refs = False
        for component in attribute_name:
            if isinstance(component, Ref):
                self._attribute_name_has_refs = True
            elif not isinstance(component, str):
                raise ValueError("The attribute name cannot have a {} component".format(component.__class__.__name__))

        self._resource = resource
        self._attribute_name = list(attribute_name)

    def as_yaml_node(self, dumper):
        name = dumper.cfn_stack.get_logical_name(self._resource, resources_only=True)  # Pass error through

        if self._attribute_name_has_refs:
            return dumper.represent_dict({
                "Fn::GetAtt": [name] + self._attribute_name
            })
        else:
            return dumper.represent_scalar("!GetAtt", ".".join([name] + self._attribute_name), style="")


class GetAZs(_Function):
    """Models the behaviour of Fn::GetAZs for Python objects.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-getavailabilityzones.html
    """

    def __init__(self, region=""):
        if region == "":
            region = Ref(AWS_Region)
        self._region = region

    def as_yaml_node(self, dumper):
        return self._get_string_node(dumper, self._region, "GetAZs")


class ImportValue(_Function):
    """Models the behaviour of Fn::ImportValue for Python objects.

    See https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-importvalue.html
    """

    def __init__(self, export_name):
        self._export_name = export_name

    def as_yaml_node(self, dumper):
        return self._get_string_node(dumper, self._export_name, "ImportValue")


class Join(_Function):
    """Models the behaviour of Fn::Join for Python objects.

    See https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-join.html
    """

    def __init__(self, delimiter, *values):
        """Join several string-like objects together at template execution time.

        Args:
            delimiter: The string used to join the values. May be the empty
                string.
            *values: The values to join together, expressed as either
                sequential parameters, or a single list. May contain other
                intrinsic functions.
        """
        # Check delimiter
        if not isinstance(delimiter, str):
            raise TypeError("Delimiter must be a string")
        if len(delimiter) > 30:
            # CloudFormation does not specify any inherent restriction on
            # delimiter length, but any large delimiter probably represents
            # a coding error
            raise ValueError("Delimiter is very large {}".format(len(delimiter)))

        # Clean values
        if len(values) == 1 and isinstance(values[0], list):
            values = list(values[0])
        else:
            values = list(values)

        if len(values) < 2:
            raise ValueError("`values` parameter must contain at least 2 values to join together")

        # Save fields for YAML output
        self._delimiter = delimiter
        self._values = list(values)

    def as_yaml_node(self, dumper):
        # TODO The default block layout is ugly. Better to use a compact form if values are not too large
        return dumper.represent_sequence("!Join", [self._delimiter, self._values])


class Ref(_Function):
    """Models the behaviour of Ref for Python objects.

    See http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-ref.html
    """

    def __init__(self, data):
        """Create a reference to a Resource or Parameter.

        :param data: The Python object that is being referenced.
        """
        # Explicitly reject attempts to use a string name directly, and
        # provide a helpful error message. Beware that PseudoParameters
        # are valid referents, but still look like a string!
        if isinstance(data, str) and not isinstance(data, PseudoParameter):
            raise TypeError("You can't directly create a Ref to a name. Try Ref._for_name(name).")

        self._data = data

    def as_yaml_node(self, dumper):
        name = dumper.cfn_stack.get_logical_name(self._data)  # Pass error through
        return dumper.represent_scalar("!Ref", name, style="")

    @classmethod
    def _for_name(cls, name):
        # TODO doco - you can use this, but better not to
        # TODO implement as a private subclass. Still get type safety, but shortcut the lookup process
        assert False

    def __eq__(self, other):
        # noinspection PyProtectedMember
        if not isinstance(other, self.__class__):
            return False

        # Two references are equal if they refer to *exactly* the same object
        return self._data is other._data

    def __hash__(self):
        # An immutable class that implements equality should also implement hash.
        # Equal objects should have the same hash, so we derive our has from
        # the class and the referred object
        return hash((self.__class__, id(self._data)))


class Sub(_Function):
    """Models the behaviour of Ref for Python objects.

    See https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-sub.html
    """

    def __init__(self, input: str, **vars):
        """Create a string that will have values substituted in
        by the CloudFormation service

        Args:
            input: Sub-specific string containing "${Name}" -style references
                to values in this CloudFormation template.
            vars: (Optional) Mapping of {name: value} for substituted values
                referenced by the input string, that are not otherwise present
                in this template.
        """
        # Checks
        if not isinstance(input, str):
            raise TypeError("The Fn::Sub function can only accept a string.")
        if isinstance(input, PseudoParameter):
            # Beware that a PseudoParameter is actually a string, in Flying Circus
            raise TypeError("Don't try to use a Pseudo Parameter as the input of Fn::Sub.")

        for name in vars.keys():
            if not isinstance(name, str):
                raise TypeError(f"Variable names for Fn::Sub function must be a string, not `{name}`.")

        self._input = input
        self._variables = vars

    def as_yaml_node(self, dumper):
        if self._variables:
            # Use long form
            return dumper.represent_sequence("!Sub", [self._input, self._variables])

        # Use short form without any extra named variables.
        #
        # Strings used in Sub tend to be full of special characters, so we
        # force them to be quoted in order to improve readability.

        # TODO not sure this forced quoting makes sense. Maybe revisit with some concrete examples.
        # If we do want it, we need to apply it to the long form as well
        return represent_string(dumper, self._input, tag="!Sub", basicsep="'")



"""General-use classes to interact with the CodeDeploy service through CloudFormation.

See Also:
    `AWS developer guide for CodeDeploy
    <https://docs.aws.amazon.com/codedeploy/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import codedeploy as _raw

# noinspection PyUnresolvedReferences
from .._raw.codedeploy import *



"""General-use classes to interact with the WorkSpaces service through CloudFormation.

See Also:
    `AWS developer guide for WorkSpaces
    <https://docs.aws.amazon.com/workspaces/latest/adminguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import workspaces as _raw

# noinspection PyUnresolvedReferences
from .._raw.workspaces import *



"""Raw representations of every data type in the AWS ElasticLoadBalancingV2 service.

See Also:
    `AWS developer guide for ElasticLoadBalancingV2
    <https://docs.aws.amazon.com/elasticloadbalancing/latest/application/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Listener",
    "ListenerProperties",
    "ListenerCertificate",
    "ListenerCertificateProperties",
    "ListenerRule",
    "ListenerRuleProperties",
    "LoadBalancer",
    "LoadBalancerProperties",
    "TargetGroup",
    "TargetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class ListenerProperties(ResourceProperties):
    Certificates = attrib(default=None)
    DefaultActions = attrib(default=None)
    LoadBalancerArn = attrib(default=None)
    Port = attrib(default=None)
    Protocol = attrib(default=None)
    SslPolicy = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Listener(Resource):
    """A Listener for ElasticLoadBalancingV2.

    See Also:
        `AWS Cloud Formation documentation for Listener
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancingV2::Listener"

    Properties: ListenerProperties = attrib(
        factory=ListenerProperties,
        converter=create_object_converter(ListenerProperties),
    )


@attrs(**ATTRSCONFIG)
class ListenerCertificateProperties(ResourceProperties):
    Certificates = attrib(default=None)
    ListenerArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ListenerCertificate(Resource):
    """A Listener Certificate for ElasticLoadBalancingV2.

    See Also:
        `AWS Cloud Formation documentation for ListenerCertificate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenercertificate.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancingV2::ListenerCertificate"

    Properties: ListenerCertificateProperties = attrib(
        factory=ListenerCertificateProperties,
        converter=create_object_converter(ListenerCertificateProperties),
    )


@attrs(**ATTRSCONFIG)
class ListenerRuleProperties(ResourceProperties):
    Actions = attrib(default=None)
    Conditions = attrib(default=None)
    ListenerArn = attrib(default=None)
    Priority = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ListenerRule(Resource):
    """A Listener Rule for ElasticLoadBalancingV2.

    See Also:
        `AWS Cloud Formation documentation for ListenerRule
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancingV2::ListenerRule"

    Properties: ListenerRuleProperties = attrib(
        factory=ListenerRuleProperties,
        converter=create_object_converter(ListenerRuleProperties),
    )


@attrs(**ATTRSCONFIG)
class LoadBalancerProperties(ResourceProperties):
    IpAddressType = attrib(default=None)
    LoadBalancerAttributes = attrib(default=None)
    Name = attrib(default=None)
    Scheme = attrib(default=None)
    SecurityGroups = attrib(default=None)
    SubnetMappings = attrib(default=None)
    Subnets = attrib(default=None)
    Tags = attrib(default=None)
    Type = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LoadBalancer(Resource):
    """A Load Balancer for ElasticLoadBalancingV2.

    See Also:
        `AWS Cloud Formation documentation for LoadBalancer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancingV2::LoadBalancer"

    Properties: LoadBalancerProperties = attrib(
        factory=LoadBalancerProperties,
        converter=create_object_converter(LoadBalancerProperties),
    )


@attrs(**ATTRSCONFIG)
class TargetGroupProperties(ResourceProperties):
    HealthCheckEnabled = attrib(default=None)
    HealthCheckIntervalSeconds = attrib(default=None)
    HealthCheckPath = attrib(default=None)
    HealthCheckPort = attrib(default=None)
    HealthCheckProtocol = attrib(default=None)
    HealthCheckTimeoutSeconds = attrib(default=None)
    HealthyThresholdCount = attrib(default=None)
    Matcher = attrib(default=None)
    Name = attrib(default=None)
    Port = attrib(default=None)
    Protocol = attrib(default=None)
    Tags = attrib(default=None)
    TargetGroupAttributes = attrib(default=None)
    Targets = attrib(default=None)
    TargetType = attrib(default=None)
    UnhealthyThresholdCount = attrib(default=None)
    VpcId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TargetGroup(Resource):
    """A Target Group for ElasticLoadBalancingV2.

    See Also:
        `AWS Cloud Formation documentation for TargetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancingV2::TargetGroup"

    Properties: TargetGroupProperties = attrib(
        factory=TargetGroupProperties,
        converter=create_object_converter(TargetGroupProperties),
    )



"""Raw representations of every data type in the AWS Neptune service.

See Also:
    `AWS developer guide for Neptune
    <https://docs.aws.amazon.com/neptune/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "DBCluster",
    "DBClusterProperties",
    "DBClusterParameterGroup",
    "DBClusterParameterGroupProperties",
    "DBInstance",
    "DBInstanceProperties",
    "DBParameterGroup",
    "DBParameterGroupProperties",
    "DBSubnetGroup",
    "DBSubnetGroupProperties",
]


@attrs(**ATTRSCONFIG)
class DBClusterProperties(ResourceProperties):
    AvailabilityZones = attrib(default=None)
    BackupRetentionPeriod = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBClusterParameterGroupName = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    IamAuthEnabled = attrib(default=None)
    KmsKeyId = attrib(default=None)
    Port = attrib(default=None)
    PreferredBackupWindow = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    SnapshotIdentifier = attrib(default=None)
    StorageEncrypted = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBCluster(Resource):
    """A Db Cluster for Neptune.

    See Also:
        `AWS Cloud Formation documentation for DBCluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbcluster.html>`_
    """

    RESOURCE_TYPE = "AWS::Neptune::DBCluster"

    Properties: DBClusterProperties = attrib(
        factory=DBClusterProperties,
        converter=create_object_converter(DBClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Family = attrib(default=None)
    Name = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroup(Resource):
    """A Db Cluster Parameter Group for Neptune.

    See Also:
        `AWS Cloud Formation documentation for DBClusterParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbclusterparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Neptune::DBClusterParameterGroup"

    Properties: DBClusterParameterGroupProperties = attrib(
        factory=DBClusterParameterGroupProperties,
        converter=create_object_converter(DBClusterParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBInstanceProperties(ResourceProperties):
    AllowMajorVersionUpgrade = attrib(default=None)
    AutoMinorVersionUpgrade = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBInstanceClass = attrib(default=None)
    DBInstanceIdentifier = attrib(default=None)
    DBParameterGroupName = attrib(default=None)
    DBSnapshotIdentifier = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBInstance(Resource):
    """A Db Instance for Neptune.

    See Also:
        `AWS Cloud Formation documentation for DBInstance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbinstance.html>`_
    """

    RESOURCE_TYPE = "AWS::Neptune::DBInstance"

    Properties: DBInstanceProperties = attrib(
        factory=DBInstanceProperties,
        converter=create_object_converter(DBInstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class DBParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Family = attrib(default=None)
    Name = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBParameterGroup(Resource):
    """A Db Parameter Group for Neptune.

    See Also:
        `AWS Cloud Formation documentation for DBParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Neptune::DBParameterGroup"

    Properties: DBParameterGroupProperties = attrib(
        factory=DBParameterGroupProperties,
        converter=create_object_converter(DBParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBSubnetGroupProperties(ResourceProperties):
    DBSubnetGroupDescription = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBSubnetGroup(Resource):
    """A Db Subnet Group for Neptune.

    See Also:
        `AWS Cloud Formation documentation for DBSubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-neptune-dbsubnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::Neptune::DBSubnetGroup"

    Properties: DBSubnetGroupProperties = attrib(
        factory=DBSubnetGroupProperties,
        converter=create_object_converter(DBSubnetGroupProperties),
    )



"""General-use classes to interact with the Lambda service through CloudFormation.

See Also:
    `AWS developer guide for Lambda
    <https://docs.aws.amazon.com/lambda/latest/dg/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import lambda_ as _raw

# noinspection PyUnresolvedReferences
from .._raw.lambda_ import *



"""Raw representations of every data type in the AWS CodePipeline service.

See Also:
    `AWS developer guide for CodePipeline
    <https://docs.aws.amazon.com/codepipeline/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "CustomActionType",
    "CustomActionTypeProperties",
    "Pipeline",
    "PipelineProperties",
    "Webhook",
    "WebhookProperties",
]


@attrs(**ATTRSCONFIG)
class CustomActionTypeProperties(ResourceProperties):
    Category = attrib(default=None)
    ConfigurationProperties = attrib(default=None)
    InputArtifactDetails = attrib(default=None)
    OutputArtifactDetails = attrib(default=None)
    Provider = attrib(default=None)
    Settings = attrib(default=None)
    Version = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CustomActionType(Resource):
    """A Custom Action Type for CodePipeline.

    See Also:
        `AWS Cloud Formation documentation for CustomActionType
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html>`_
    """

    RESOURCE_TYPE = "AWS::CodePipeline::CustomActionType"

    Properties: CustomActionTypeProperties = attrib(
        factory=CustomActionTypeProperties,
        converter=create_object_converter(CustomActionTypeProperties),
    )


@attrs(**ATTRSCONFIG)
class PipelineProperties(ResourceProperties):
    ArtifactStore = attrib(default=None)
    ArtifactStores = attrib(default=None)
    DisableInboundStageTransitions = attrib(default=None)
    Name = attrib(default=None)
    RestartExecutionOnUpdate = attrib(default=None)
    RoleArn = attrib(default=None)
    Stages = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Pipeline(Resource):
    """A Pipeline for CodePipeline.

    See Also:
        `AWS Cloud Formation documentation for Pipeline
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html>`_
    """

    RESOURCE_TYPE = "AWS::CodePipeline::Pipeline"

    Properties: PipelineProperties = attrib(
        factory=PipelineProperties,
        converter=create_object_converter(PipelineProperties),
    )


@attrs(**ATTRSCONFIG)
class WebhookProperties(ResourceProperties):
    Authentication = attrib(default=None)
    AuthenticationConfiguration = attrib(default=None)
    Filters = attrib(default=None)
    Name = attrib(default=None)
    RegisterWithThirdParty = attrib(default=None)
    TargetAction = attrib(default=None)
    TargetPipeline = attrib(default=None)
    TargetPipelineVersion = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Webhook(Resource):
    """A Webhook for CodePipeline.

    See Also:
        `AWS Cloud Formation documentation for Webhook
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-webhook.html>`_
    """

    RESOURCE_TYPE = "AWS::CodePipeline::Webhook"

    Properties: WebhookProperties = attrib(
        factory=WebhookProperties,
        converter=create_object_converter(WebhookProperties),
    )



"""Raw representations of every data type in the AWS ServiceDiscovery service.

See Also:
    `AWS developer guide for ServiceDiscovery
    <https://docs.aws.amazon.com/cloud-map/latest/dg/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "HttpNamespace",
    "HttpNamespaceProperties",
    "Instance",
    "InstanceProperties",
    "PrivateDnsNamespace",
    "PrivateDnsNamespaceProperties",
    "PublicDnsNamespace",
    "PublicDnsNamespaceProperties",
    "Service",
    "ServiceProperties",
]


@attrs(**ATTRSCONFIG)
class HttpNamespaceProperties(ResourceProperties):
    Description = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class HttpNamespace(Resource):
    """A Http Namespace for ServiceDiscovery.

    See Also:
        `AWS Cloud Formation documentation for HttpNamespace
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicediscovery-httpnamespace.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceDiscovery::HttpNamespace"

    Properties: HttpNamespaceProperties = attrib(
        factory=HttpNamespaceProperties,
        converter=create_object_converter(HttpNamespaceProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceProperties(ResourceProperties):
    InstanceAttributes = attrib(default=None)
    InstanceId = attrib(default=None)
    ServiceId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Instance(Resource):
    """A Instance for ServiceDiscovery.

    See Also:
        `AWS Cloud Formation documentation for Instance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicediscovery-instance.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceDiscovery::Instance"

    Properties: InstanceProperties = attrib(
        factory=InstanceProperties,
        converter=create_object_converter(InstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class PrivateDnsNamespaceProperties(ResourceProperties):
    Description = attrib(default=None)
    Name = attrib(default=None)
    Vpc = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PrivateDnsNamespace(Resource):
    """A Private Dns Namespace for ServiceDiscovery.

    See Also:
        `AWS Cloud Formation documentation for PrivateDnsNamespace
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicediscovery-privatednsnamespace.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceDiscovery::PrivateDnsNamespace"

    Properties: PrivateDnsNamespaceProperties = attrib(
        factory=PrivateDnsNamespaceProperties,
        converter=create_object_converter(PrivateDnsNamespaceProperties),
    )


@attrs(**ATTRSCONFIG)
class PublicDnsNamespaceProperties(ResourceProperties):
    Description = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PublicDnsNamespace(Resource):
    """A Public Dns Namespace for ServiceDiscovery.

    See Also:
        `AWS Cloud Formation documentation for PublicDnsNamespace
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicediscovery-publicdnsnamespace.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceDiscovery::PublicDnsNamespace"

    Properties: PublicDnsNamespaceProperties = attrib(
        factory=PublicDnsNamespaceProperties,
        converter=create_object_converter(PublicDnsNamespaceProperties),
    )


@attrs(**ATTRSCONFIG)
class ServiceProperties(ResourceProperties):
    Description = attrib(default=None)
    DnsConfig = attrib(default=None)
    HealthCheckConfig = attrib(default=None)
    HealthCheckCustomConfig = attrib(default=None)
    Name = attrib(default=None)
    NamespaceId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Service(Resource):
    """A Service for ServiceDiscovery.

    See Also:
        `AWS Cloud Formation documentation for Service
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-servicediscovery-service.html>`_
    """

    RESOURCE_TYPE = "AWS::ServiceDiscovery::Service"

    Properties: ServiceProperties = attrib(
        factory=ServiceProperties,
        converter=create_object_converter(ServiceProperties),
    )



"""General-use classes to interact with the CloudFormation service through CloudFormation.

See Also:
    `AWS developer guide for CloudFormation
    <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import cloudformation as _raw

# noinspection PyUnresolvedReferences
from .._raw.cloudformation import *



"""Raw representations of every data type in the AWS RDS service.

See Also:
    `AWS developer guide for RDS
    <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "DBCluster",
    "DBClusterProperties",
    "DBClusterParameterGroup",
    "DBClusterParameterGroupProperties",
    "DBInstance",
    "DBInstanceProperties",
    "DBParameterGroup",
    "DBParameterGroupProperties",
    "DBSecurityGroup",
    "DBSecurityGroupProperties",
    "DBSecurityGroupIngress",
    "DBSecurityGroupIngressProperties",
    "DBSubnetGroup",
    "DBSubnetGroupProperties",
    "EventSubscription",
    "EventSubscriptionProperties",
    "OptionGroup",
    "OptionGroupProperties",
]


@attrs(**ATTRSCONFIG)
class DBClusterProperties(ResourceProperties):
    AvailabilityZones = attrib(default=None)
    BacktrackWindow = attrib(default=None)
    BackupRetentionPeriod = attrib(default=None)
    DatabaseName = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBClusterParameterGroupName = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    DeletionProtection = attrib(default=None)
    EnableCloudwatchLogsExports = attrib(default=None)
    EnableIAMDatabaseAuthentication = attrib(default=None)
    Engine = attrib(default=None)
    EngineMode = attrib(default=None)
    EngineVersion = attrib(default=None)
    KmsKeyId = attrib(default=None)
    MasterUsername = attrib(default=None)
    MasterUserPassword = attrib(default=None)
    Port = attrib(default=None)
    PreferredBackupWindow = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    ReplicationSourceIdentifier = attrib(default=None)
    ScalingConfiguration = attrib(default=None)
    SnapshotIdentifier = attrib(default=None)
    SourceRegion = attrib(default=None)
    StorageEncrypted = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBCluster(Resource):
    """A Db Cluster for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBCluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBCluster"

    Properties: DBClusterProperties = attrib(
        factory=DBClusterProperties,
        converter=create_object_converter(DBClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Family = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBClusterParameterGroup(Resource):
    """A Db Cluster Parameter Group for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBClusterParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBClusterParameterGroup"

    Properties: DBClusterParameterGroupProperties = attrib(
        factory=DBClusterParameterGroupProperties,
        converter=create_object_converter(DBClusterParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBInstanceProperties(ResourceProperties):
    AllocatedStorage = attrib(default=None)
    AllowMajorVersionUpgrade = attrib(default=None)
    AutoMinorVersionUpgrade = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    BackupRetentionPeriod = attrib(default=None)
    CharacterSetName = attrib(default=None)
    CopyTagsToSnapshot = attrib(default=None)
    DBClusterIdentifier = attrib(default=None)
    DBInstanceClass = attrib(default=None)
    DBInstanceIdentifier = attrib(default=None)
    DBName = attrib(default=None)
    DBParameterGroupName = attrib(default=None)
    DBSecurityGroups = attrib(default=None)
    DBSnapshotIdentifier = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    DeleteAutomatedBackups = attrib(default=None)
    DeletionProtection = attrib(default=None)
    Domain = attrib(default=None)
    DomainIAMRoleName = attrib(default=None)
    EnableCloudwatchLogsExports = attrib(default=None)
    EnableIAMDatabaseAuthentication = attrib(default=None)
    EnablePerformanceInsights = attrib(default=None)
    Engine = attrib(default=None)
    EngineVersion = attrib(default=None)
    Iops = attrib(default=None)
    KmsKeyId = attrib(default=None)
    LicenseModel = attrib(default=None)
    MasterUsername = attrib(default=None)
    MasterUserPassword = attrib(default=None)
    MonitoringInterval = attrib(default=None)
    MonitoringRoleArn = attrib(default=None)
    MultiAZ = attrib(default=None)
    OptionGroupName = attrib(default=None)
    PerformanceInsightsKMSKeyId = attrib(default=None)
    PerformanceInsightsRetentionPeriod = attrib(default=None)
    Port = attrib(default=None)
    PreferredBackupWindow = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    ProcessorFeatures = attrib(default=None)
    PromotionTier = attrib(default=None)
    PubliclyAccessible = attrib(default=None)
    SourceDBInstanceIdentifier = attrib(default=None)
    SourceRegion = attrib(default=None)
    StorageEncrypted = attrib(default=None)
    StorageType = attrib(default=None)
    Tags = attrib(default=None)
    Timezone = attrib(default=None)
    UseDefaultProcessorFeatures = attrib(default=None)
    VPCSecurityGroups = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBInstance(Resource):
    """A Db Instance for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBInstance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBInstance"

    Properties: DBInstanceProperties = attrib(
        factory=DBInstanceProperties,
        converter=create_object_converter(DBInstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class DBParameterGroupProperties(ResourceProperties):
    Description = attrib(default=None)
    Family = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBParameterGroup(Resource):
    """A Db Parameter Group for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBParameterGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBParameterGroup"

    Properties: DBParameterGroupProperties = attrib(
        factory=DBParameterGroupProperties,
        converter=create_object_converter(DBParameterGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBSecurityGroupProperties(ResourceProperties):
    DBSecurityGroupIngress = attrib(default=None)
    EC2VpcId = attrib(default=None)
    GroupDescription = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBSecurityGroup(Resource):
    """A Db Security Group for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBSecurityGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBSecurityGroup"

    Properties: DBSecurityGroupProperties = attrib(
        factory=DBSecurityGroupProperties,
        converter=create_object_converter(DBSecurityGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class DBSecurityGroupIngressProperties(ResourceProperties):
    CIDRIP = attrib(default=None)
    DBSecurityGroupName = attrib(default=None)
    EC2SecurityGroupId = attrib(default=None)
    EC2SecurityGroupName = attrib(default=None)
    EC2SecurityGroupOwnerId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBSecurityGroupIngress(Resource):
    """A Db Security Group Ingress for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBSecurityGroupIngress
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBSecurityGroupIngress"

    Properties: DBSecurityGroupIngressProperties = attrib(
        factory=DBSecurityGroupIngressProperties,
        converter=create_object_converter(DBSecurityGroupIngressProperties),
    )


@attrs(**ATTRSCONFIG)
class DBSubnetGroupProperties(ResourceProperties):
    DBSubnetGroupDescription = attrib(default=None)
    DBSubnetGroupName = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DBSubnetGroup(Resource):
    """A Db Subnet Group for RDS.

    See Also:
        `AWS Cloud Formation documentation for DBSubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::DBSubnetGroup"

    Properties: DBSubnetGroupProperties = attrib(
        factory=DBSubnetGroupProperties,
        converter=create_object_converter(DBSubnetGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class EventSubscriptionProperties(ResourceProperties):
    Enabled = attrib(default=None)
    EventCategories = attrib(default=None)
    SnsTopicArn = attrib(default=None)
    SourceIds = attrib(default=None)
    SourceType = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EventSubscription(Resource):
    """A Event Subscription for RDS.

    See Also:
        `AWS Cloud Formation documentation for EventSubscription
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::EventSubscription"

    Properties: EventSubscriptionProperties = attrib(
        factory=EventSubscriptionProperties,
        converter=create_object_converter(EventSubscriptionProperties),
    )


@attrs(**ATTRSCONFIG)
class OptionGroupProperties(ResourceProperties):
    EngineName = attrib(default=None)
    MajorEngineVersion = attrib(default=None)
    OptionConfigurations = attrib(default=None)
    OptionGroupDescription = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class OptionGroup(Resource):
    """A Option Group for RDS.

    See Also:
        `AWS Cloud Formation documentation for OptionGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html>`_
    """

    RESOURCE_TYPE = "AWS::RDS::OptionGroup"

    Properties: OptionGroupProperties = attrib(
        factory=OptionGroupProperties,
        converter=create_object_converter(OptionGroupProperties),
    )



"""General-use classes to interact with the CodePipeline service through CloudFormation.

See Also:
    `AWS developer guide for CodePipeline
    <https://docs.aws.amazon.com/codepipeline/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import codepipeline as _raw

# noinspection PyUnresolvedReferences
from .._raw.codepipeline import *



"""General-use classes to interact with the Budgets service through CloudFormation.

See Also:
    `AWS developer guide for Budgets
    <https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import budgets as _raw

# noinspection PyUnresolvedReferences
from .._raw.budgets import *



"""General-use classes to interact with the Logs service through CloudFormation.

See Also:
    `AWS developer guide for Logs
    <https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import logs as _raw

# noinspection PyUnresolvedReferences
from .._raw.logs import *



"""General-use classes to interact with the OpsWorksCM service through CloudFormation.

See Also:
    `AWS developer guide for OpsWorksCM
    <https://docs.aws.amazon.com/opsworks/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import opsworkscm as _raw

# noinspection PyUnresolvedReferences
from .._raw.opsworkscm import *



"""General-use classes to interact with the Cognito service through CloudFormation.

See Also:
    `AWS developer guide for Cognito
    <https://docs.aws.amazon.com/cognito/latest/developerguide/index.html>`_
"""

from .._raw import cognito as _raw

# noinspection PyUnresolvedReferences
from .._raw.cognito import *


class UserPool(_raw.UserPool):
    __slots__ = []

    # Implement Tagging
    # -----------------
    #
    # User Pools store tags as a dictionary (somebody had to), and use a
    # different Property name for the tag data
    TAG_PROPERTY = "UserPoolTags"

    def tag(self, tags=None, tag_derived_resources=True, **more_tags):
        tagdata = self.Properties[self.TAG_PROPERTY]

        if tagdata is None:
            # No tags set yet
            self.Properties[self.TAG_PROPERTY] = {}
            tagdata = self.Properties[self.TAG_PROPERTY]

        tagdata.update(tags or {})
        tagdata.update(more_tags)

        return True

    def get_tag(self, key: str):
        tagdata = self.Properties[self.TAG_PROPERTY]

        if tagdata is None:
            # No tags set yet
            return None

        return tagdata.get(key, None)

    # Implement Naming
    # ----------------
    @property
    def name(self):
        return self.Properties["UserPoolName"]  # Might be default of None

    @name.setter
    def name(self, value: str):
        self.Properties["UserPoolName"] = value



"""Raw representations of every data type in the AWS DMS service.

See Also:
    `AWS developer guide for DMS
    <https://docs.aws.amazon.com/dms/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Certificate",
    "CertificateProperties",
    "Endpoint",
    "EndpointProperties",
    "EventSubscription",
    "EventSubscriptionProperties",
    "ReplicationInstance",
    "ReplicationInstanceProperties",
    "ReplicationSubnetGroup",
    "ReplicationSubnetGroupProperties",
    "ReplicationTask",
    "ReplicationTaskProperties",
]


@attrs(**ATTRSCONFIG)
class CertificateProperties(ResourceProperties):
    CertificateIdentifier = attrib(default=None)
    CertificatePem = attrib(default=None)
    CertificateWallet = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Certificate(Resource):
    """A Certificate for DMS.

    See Also:
        `AWS Cloud Formation documentation for Certificate
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::Certificate"

    Properties: CertificateProperties = attrib(
        factory=CertificateProperties,
        converter=create_object_converter(CertificateProperties),
    )


@attrs(**ATTRSCONFIG)
class EndpointProperties(ResourceProperties):
    CertificateArn = attrib(default=None)
    DatabaseName = attrib(default=None)
    DynamoDbSettings = attrib(default=None)
    ElasticsearchSettings = attrib(default=None)
    EndpointIdentifier = attrib(default=None)
    EndpointType = attrib(default=None)
    EngineName = attrib(default=None)
    ExtraConnectionAttributes = attrib(default=None)
    KinesisSettings = attrib(default=None)
    KmsKeyId = attrib(default=None)
    MongoDbSettings = attrib(default=None)
    Password = attrib(default=None)
    Port = attrib(default=None)
    S3Settings = attrib(default=None)
    ServerName = attrib(default=None)
    SslMode = attrib(default=None)
    Tags = attrib(default=None)
    Username = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Endpoint(Resource):
    """A Endpoint for DMS.

    See Also:
        `AWS Cloud Formation documentation for Endpoint
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::Endpoint"

    Properties: EndpointProperties = attrib(
        factory=EndpointProperties,
        converter=create_object_converter(EndpointProperties),
    )


@attrs(**ATTRSCONFIG)
class EventSubscriptionProperties(ResourceProperties):
    Enabled = attrib(default=None)
    EventCategories = attrib(default=None)
    SnsTopicArn = attrib(default=None)
    SourceIds = attrib(default=None)
    SourceType = attrib(default=None)
    SubscriptionName = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class EventSubscription(Resource):
    """A Event Subscription for DMS.

    See Also:
        `AWS Cloud Formation documentation for EventSubscription
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::EventSubscription"

    Properties: EventSubscriptionProperties = attrib(
        factory=EventSubscriptionProperties,
        converter=create_object_converter(EventSubscriptionProperties),
    )


@attrs(**ATTRSCONFIG)
class ReplicationInstanceProperties(ResourceProperties):
    AllocatedStorage = attrib(default=None)
    AllowMajorVersionUpgrade = attrib(default=None)
    AutoMinorVersionUpgrade = attrib(default=None)
    AvailabilityZone = attrib(default=None)
    EngineVersion = attrib(default=None)
    KmsKeyId = attrib(default=None)
    MultiAZ = attrib(default=None)
    PreferredMaintenanceWindow = attrib(default=None)
    PubliclyAccessible = attrib(default=None)
    ReplicationInstanceClass = attrib(default=None)
    ReplicationInstanceIdentifier = attrib(default=None)
    ReplicationSubnetGroupIdentifier = attrib(default=None)
    Tags = attrib(default=None)
    VpcSecurityGroupIds = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReplicationInstance(Resource):
    """A Replication Instance for DMS.

    See Also:
        `AWS Cloud Formation documentation for ReplicationInstance
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::ReplicationInstance"

    Properties: ReplicationInstanceProperties = attrib(
        factory=ReplicationInstanceProperties,
        converter=create_object_converter(ReplicationInstanceProperties),
    )


@attrs(**ATTRSCONFIG)
class ReplicationSubnetGroupProperties(ResourceProperties):
    ReplicationSubnetGroupDescription = attrib(default=None)
    ReplicationSubnetGroupIdentifier = attrib(default=None)
    SubnetIds = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReplicationSubnetGroup(Resource):
    """A Replication Subnet Group for DMS.

    See Also:
        `AWS Cloud Formation documentation for ReplicationSubnetGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::ReplicationSubnetGroup"

    Properties: ReplicationSubnetGroupProperties = attrib(
        factory=ReplicationSubnetGroupProperties,
        converter=create_object_converter(ReplicationSubnetGroupProperties),
    )


@attrs(**ATTRSCONFIG)
class ReplicationTaskProperties(ResourceProperties):
    CdcStartTime = attrib(default=None)
    MigrationType = attrib(default=None)
    ReplicationInstanceArn = attrib(default=None)
    ReplicationTaskIdentifier = attrib(default=None)
    ReplicationTaskSettings = attrib(default=None)
    SourceEndpointArn = attrib(default=None)
    TableMappings = attrib(default=None)
    Tags = attrib(default=None)
    TargetEndpointArn = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ReplicationTask(Resource):
    """A Replication Task for DMS.

    See Also:
        `AWS Cloud Formation documentation for ReplicationTask
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html>`_
    """

    RESOURCE_TYPE = "AWS::DMS::ReplicationTask"

    Properties: ReplicationTaskProperties = attrib(
        factory=ReplicationTaskProperties,
        converter=create_object_converter(ReplicationTaskProperties),
    )



"""General-use classes to interact with the KinesisAnalyticsV2 service through CloudFormation.

See Also:
    `AWS developer guide for KinesisAnalyticsV2
    <https://docs.aws.amazon.com/kinesisanalytics/latest/java/what-is.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import kinesisanalyticsv2 as _raw

# noinspection PyUnresolvedReferences
from .._raw.kinesisanalyticsv2 import *



"""Raw representations of every data type in the AWS AppStream service.

See Also:
    `AWS developer guide for AppStream
    <https://docs.aws.amazon.com/appstream2/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "DirectoryConfig",
    "DirectoryConfigProperties",
    "Fleet",
    "FleetProperties",
    "ImageBuilder",
    "ImageBuilderProperties",
    "Stack",
    "StackProperties",
    "StackFleetAssociation",
    "StackFleetAssociationProperties",
    "StackUserAssociation",
    "StackUserAssociationProperties",
    "User",
    "UserProperties",
]


@attrs(**ATTRSCONFIG)
class DirectoryConfigProperties(ResourceProperties):
    DirectoryName = attrib(default=None)
    OrganizationalUnitDistinguishedNames = attrib(default=None)
    ServiceAccountCredentials = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DirectoryConfig(Resource):
    """A Directory Config for AppStream.

    See Also:
        `AWS Cloud Formation documentation for DirectoryConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-directoryconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::DirectoryConfig"

    Properties: DirectoryConfigProperties = attrib(
        factory=DirectoryConfigProperties,
        converter=create_object_converter(DirectoryConfigProperties),
    )


@attrs(**ATTRSCONFIG)
class FleetProperties(ResourceProperties):
    ComputeCapacity = attrib(default=None)
    Description = attrib(default=None)
    DisconnectTimeoutInSeconds = attrib(default=None)
    DisplayName = attrib(default=None)
    DomainJoinInfo = attrib(default=None)
    EnableDefaultInternetAccess = attrib(default=None)
    FleetType = attrib(default=None)
    ImageArn = attrib(default=None)
    ImageName = attrib(default=None)
    InstanceType = attrib(default=None)
    MaxUserDurationInSeconds = attrib(default=None)
    Name = attrib(default=None)
    VpcConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Fleet(Resource):
    """A Fleet for AppStream.

    See Also:
        `AWS Cloud Formation documentation for Fleet
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-fleet.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::Fleet"

    Properties: FleetProperties = attrib(
        factory=FleetProperties,
        converter=create_object_converter(FleetProperties),
    )


@attrs(**ATTRSCONFIG)
class ImageBuilderProperties(ResourceProperties):
    AppstreamAgentVersion = attrib(default=None)
    Description = attrib(default=None)
    DisplayName = attrib(default=None)
    DomainJoinInfo = attrib(default=None)
    EnableDefaultInternetAccess = attrib(default=None)
    ImageArn = attrib(default=None)
    ImageName = attrib(default=None)
    InstanceType = attrib(default=None)
    Name = attrib(default=None)
    VpcConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ImageBuilder(Resource):
    """A Image Builder for AppStream.

    See Also:
        `AWS Cloud Formation documentation for ImageBuilder
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-imagebuilder.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::ImageBuilder"

    Properties: ImageBuilderProperties = attrib(
        factory=ImageBuilderProperties,
        converter=create_object_converter(ImageBuilderProperties),
    )


@attrs(**ATTRSCONFIG)
class StackProperties(ResourceProperties):
    ApplicationSettings = attrib(default=None)
    AttributesToDelete = attrib(default=None)
    DeleteStorageConnectors = attrib(default=None)
    Description = attrib(default=None)
    DisplayName = attrib(default=None)
    FeedbackURL = attrib(default=None)
    Name = attrib(default=None)
    RedirectURL = attrib(default=None)
    StorageConnectors = attrib(default=None)
    UserSettings = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stack(Resource):
    """A Stack for AppStream.

    See Also:
        `AWS Cloud Formation documentation for Stack
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stack.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::Stack"

    Properties: StackProperties = attrib(
        factory=StackProperties,
        converter=create_object_converter(StackProperties),
    )


@attrs(**ATTRSCONFIG)
class StackFleetAssociationProperties(ResourceProperties):
    FleetName = attrib(default=None)
    StackName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class StackFleetAssociation(Resource):
    """A Stack Fleet Association for AppStream.

    See Also:
        `AWS Cloud Formation documentation for StackFleetAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stackfleetassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::StackFleetAssociation"

    Properties: StackFleetAssociationProperties = attrib(
        factory=StackFleetAssociationProperties,
        converter=create_object_converter(StackFleetAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class StackUserAssociationProperties(ResourceProperties):
    AuthenticationType = attrib(default=None)
    SendEmailNotification = attrib(default=None)
    StackName = attrib(default=None)
    UserName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class StackUserAssociation(Resource):
    """A Stack User Association for AppStream.

    See Also:
        `AWS Cloud Formation documentation for StackUserAssociation
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-stackuserassociation.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::StackUserAssociation"

    Properties: StackUserAssociationProperties = attrib(
        factory=StackUserAssociationProperties,
        converter=create_object_converter(StackUserAssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class UserProperties(ResourceProperties):
    AuthenticationType = attrib(default=None)
    FirstName = attrib(default=None)
    LastName = attrib(default=None)
    MessageAction = attrib(default=None)
    UserName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class User(Resource):
    """A User for AppStream.

    See Also:
        `AWS Cloud Formation documentation for User
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-appstream-user.html>`_
    """

    RESOURCE_TYPE = "AWS::AppStream::User"

    Properties: UserProperties = attrib(
        factory=UserProperties,
        converter=create_object_converter(UserProperties),
    )



"""Raw representations of every data type in the AWS ECS service.

See Also:
    `AWS developer guide for ECS
    <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Cluster",
    "ClusterProperties",
    "Service",
    "ServiceProperties",
    "TaskDefinition",
    "TaskDefinitionProperties",
]


@attrs(**ATTRSCONFIG)
class ClusterProperties(ResourceProperties):
    ClusterName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Cluster(Resource):
    """A Cluster for ECS.

    See Also:
        `AWS Cloud Formation documentation for Cluster
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-cluster.html>`_
    """

    RESOURCE_TYPE = "AWS::ECS::Cluster"

    Properties: ClusterProperties = attrib(
        factory=ClusterProperties,
        converter=create_object_converter(ClusterProperties),
    )


@attrs(**ATTRSCONFIG)
class ServiceProperties(ResourceProperties):
    Cluster = attrib(default=None)
    DeploymentConfiguration = attrib(default=None)
    DesiredCount = attrib(default=None)
    HealthCheckGracePeriodSeconds = attrib(default=None)
    LaunchType = attrib(default=None)
    LoadBalancers = attrib(default=None)
    NetworkConfiguration = attrib(default=None)
    PlacementConstraints = attrib(default=None)
    PlacementStrategies = attrib(default=None)
    PlatformVersion = attrib(default=None)
    Role = attrib(default=None)
    SchedulingStrategy = attrib(default=None)
    ServiceName = attrib(default=None)
    ServiceRegistries = attrib(default=None)
    TaskDefinition = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Service(Resource):
    """A Service for ECS.

    See Also:
        `AWS Cloud Formation documentation for Service
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html>`_
    """

    RESOURCE_TYPE = "AWS::ECS::Service"

    Properties: ServiceProperties = attrib(
        factory=ServiceProperties,
        converter=create_object_converter(ServiceProperties),
    )


@attrs(**ATTRSCONFIG)
class TaskDefinitionProperties(ResourceProperties):
    ContainerDefinitions = attrib(default=None)
    Cpu = attrib(default=None)
    ExecutionRoleArn = attrib(default=None)
    Family = attrib(default=None)
    Memory = attrib(default=None)
    NetworkMode = attrib(default=None)
    PlacementConstraints = attrib(default=None)
    RequiresCompatibilities = attrib(default=None)
    TaskRoleArn = attrib(default=None)
    Volumes = attrib(default=None)


@attrs(**ATTRSCONFIG)
class TaskDefinition(Resource):
    """A Task Definition for ECS.

    See Also:
        `AWS Cloud Formation documentation for TaskDefinition
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html>`_
    """

    RESOURCE_TYPE = "AWS::ECS::TaskDefinition"

    Properties: TaskDefinitionProperties = attrib(
        factory=TaskDefinitionProperties,
        converter=create_object_converter(TaskDefinitionProperties),
    )



"""General-use classes to interact with the DynamoDB service through CloudFormation.

See Also:
    `AWS developer guide for DynamoDB
    <https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import dynamodb as _raw

# noinspection PyUnresolvedReferences
from .._raw.dynamodb import *



"""Raw representations of every data type in the AWS SSM service.

See Also:
    `AWS developer guide for SSM
    <https://docs.aws.amazon.com/systems-manager/latest/APIReference/Welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Association",
    "AssociationProperties",
    "Document",
    "DocumentProperties",
    "MaintenanceWindow",
    "MaintenanceWindowProperties",
    "MaintenanceWindowTask",
    "MaintenanceWindowTaskProperties",
    "Parameter",
    "ParameterProperties",
    "PatchBaseline",
    "PatchBaselineProperties",
    "ResourceDataSync",
    "ResourceDataSyncProperties",
]


@attrs(**ATTRSCONFIG)
class AssociationProperties(ResourceProperties):
    AssociationName = attrib(default=None)
    DocumentVersion = attrib(default=None)
    InstanceId = attrib(default=None)
    Name = attrib(default=None)
    OutputLocation = attrib(default=None)
    Parameters = attrib(default=None)
    ScheduleExpression = attrib(default=None)
    Targets = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Association(Resource):
    """A Association for SSM.

    See Also:
        `AWS Cloud Formation documentation for Association
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::Association"

    Properties: AssociationProperties = attrib(
        factory=AssociationProperties,
        converter=create_object_converter(AssociationProperties),
    )


@attrs(**ATTRSCONFIG)
class DocumentProperties(ResourceProperties):
    Content = attrib(default=None)
    DocumentType = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Document(Resource):
    """A Document for SSM.

    See Also:
        `AWS Cloud Formation documentation for Document
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-document.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::Document"

    Properties: DocumentProperties = attrib(
        factory=DocumentProperties,
        converter=create_object_converter(DocumentProperties),
    )


@attrs(**ATTRSCONFIG)
class MaintenanceWindowProperties(ResourceProperties):
    AllowUnassociatedTargets = attrib(default=None)
    Cutoff = attrib(default=None)
    Description = attrib(default=None)
    Duration = attrib(default=None)
    EndDate = attrib(default=None)
    Name = attrib(default=None)
    Schedule = attrib(default=None)
    ScheduleTimezone = attrib(default=None)
    StartDate = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class MaintenanceWindow(Resource):
    """A Maintenance Window for SSM.

    See Also:
        `AWS Cloud Formation documentation for MaintenanceWindow
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-maintenancewindow.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::MaintenanceWindow"

    Properties: MaintenanceWindowProperties = attrib(
        factory=MaintenanceWindowProperties,
        converter=create_object_converter(MaintenanceWindowProperties),
    )


@attrs(**ATTRSCONFIG)
class MaintenanceWindowTaskProperties(ResourceProperties):
    Description = attrib(default=None)
    LoggingInfo = attrib(default=None)
    MaxConcurrency = attrib(default=None)
    MaxErrors = attrib(default=None)
    Name = attrib(default=None)
    Priority = attrib(default=None)
    ServiceRoleArn = attrib(default=None)
    Targets = attrib(default=None)
    TaskArn = attrib(default=None)
    TaskInvocationParameters = attrib(default=None)
    TaskParameters = attrib(default=None)
    TaskType = attrib(default=None)
    WindowId = attrib(default=None)


@attrs(**ATTRSCONFIG)
class MaintenanceWindowTask(Resource):
    """A Maintenance Window Task for SSM.

    See Also:
        `AWS Cloud Formation documentation for MaintenanceWindowTask
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-maintenancewindowtask.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::MaintenanceWindowTask"

    Properties: MaintenanceWindowTaskProperties = attrib(
        factory=MaintenanceWindowTaskProperties,
        converter=create_object_converter(MaintenanceWindowTaskProperties),
    )


@attrs(**ATTRSCONFIG)
class ParameterProperties(ResourceProperties):
    AllowedPattern = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    Type = attrib(default=None)
    Value = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Parameter(Resource):
    """A Parameter for SSM.

    See Also:
        `AWS Cloud Formation documentation for Parameter
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::Parameter"

    Properties: ParameterProperties = attrib(
        factory=ParameterProperties,
        converter=create_object_converter(ParameterProperties),
    )


@attrs(**ATTRSCONFIG)
class PatchBaselineProperties(ResourceProperties):
    ApprovalRules = attrib(default=None)
    ApprovedPatches = attrib(default=None)
    ApprovedPatchesComplianceLevel = attrib(default=None)
    ApprovedPatchesEnableNonSecurity = attrib(default=None)
    Description = attrib(default=None)
    GlobalFilters = attrib(default=None)
    Name = attrib(default=None)
    OperatingSystem = attrib(default=None)
    PatchGroups = attrib(default=None)
    RejectedPatches = attrib(default=None)
    RejectedPatchesAction = attrib(default=None)
    Sources = attrib(default=None)


@attrs(**ATTRSCONFIG)
class PatchBaseline(Resource):
    """A Patch Baseline for SSM.

    See Also:
        `AWS Cloud Formation documentation for PatchBaseline
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-patchbaseline.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::PatchBaseline"

    Properties: PatchBaselineProperties = attrib(
        factory=PatchBaselineProperties,
        converter=create_object_converter(PatchBaselineProperties),
    )


@attrs(**ATTRSCONFIG)
class ResourceDataSyncProperties(ResourceProperties):
    BucketName = attrib(default=None)
    BucketPrefix = attrib(default=None)
    BucketRegion = attrib(default=None)
    KMSKeyArn = attrib(default=None)
    SyncFormat = attrib(default=None)
    SyncName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ResourceDataSync(Resource):
    """A Resource Data Sync for SSM.

    See Also:
        `AWS Cloud Formation documentation for ResourceDataSync
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-resourcedatasync.html>`_
    """

    RESOURCE_TYPE = "AWS::SSM::ResourceDataSync"

    Properties: ResourceDataSyncProperties = attrib(
        factory=ResourceDataSyncProperties,
        converter=create_object_converter(ResourceDataSyncProperties),
    )



"""Raw representations of every data type in the AWS ElasticLoadBalancing service.

See Also:
    `AWS developer guide for ElasticLoadBalancing
    <https://docs.aws.amazon.com/elasticloadbalancing/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "LoadBalancer",
    "LoadBalancerProperties",
]


@attrs(**ATTRSCONFIG)
class LoadBalancerProperties(ResourceProperties):
    AccessLoggingPolicy = attrib(default=None)
    AppCookieStickinessPolicy = attrib(default=None)
    AvailabilityZones = attrib(default=None)
    ConnectionDrainingPolicy = attrib(default=None)
    ConnectionSettings = attrib(default=None)
    CrossZone = attrib(default=None)
    HealthCheck = attrib(default=None)
    Instances = attrib(default=None)
    LBCookieStickinessPolicy = attrib(default=None)
    Listeners = attrib(default=None)
    LoadBalancerName = attrib(default=None)
    Policies = attrib(default=None)
    Scheme = attrib(default=None)
    SecurityGroups = attrib(default=None)
    Subnets = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class LoadBalancer(Resource):
    """A Load Balancer for ElasticLoadBalancing.

    See Also:
        `AWS Cloud Formation documentation for LoadBalancer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html>`_
    """

    RESOURCE_TYPE = "AWS::ElasticLoadBalancing::LoadBalancer"

    Properties: LoadBalancerProperties = attrib(
        factory=LoadBalancerProperties,
        converter=create_object_converter(LoadBalancerProperties),
    )



"""General-use classes to interact with the AutoScaling service through CloudFormation.

See Also:
    `AWS developer guide for AutoScaling
     <http://docs.aws.amazon.com/autoscaling/latest/userguide/WhatIsAutoScaling.html>`_
"""

from flyingcircus import Fn
from flyingcircus.core import AWS_Region
from flyingcircus.core import Stack
from flyingcircus.core import dedent
from . import cloudwatch
# noinspection PyUnresolvedReferences
from .._raw import autoscaling as _raw
# noinspection PyUnresolvedReferences
from .._raw.autoscaling import *


def autoscaling_group_by_cpu(low=20, high=80):
    """Create an auto-scaling group that scales based on it's CPU load."""
    # TODO this is more like a recipe than a basic service
    stack = Stack(
        # TODO generate description by auto-breaking the line with the (not-yet-existent) reflow function instead
        Description=dedent("""
            Deploy an auto-scaling group that scales based on lower and upper CPU usage
            thresholds.
            """),
    )

    launch_config = stack.Resources["LaunchConfiguration"] = LaunchConfiguration(
        Properties=LaunchConfigurationProperties(
            ImageId="ami-1a668878",  # Amazon Linux 2017.09.01 in ap-southeast-2
            InstanceType="t2.micro",  # TODO consider making this a lookup value
            # TODO KeyName would probably be helpful
        ),
    )

    asg = stack.Resources["AutoScalingGroup"] = AutoScalingGroup(
        Properties=AutoScalingGroupProperties(
            AvailabilityZones=Fn.GetAZs(Fn.Ref(AWS_Region)),
            LaunchConfigurationName=Fn.Ref(launch_config),
            MinSize=1,
            MaxSize=3,
        ),
    )

    stack.merge_stack(
        simple_scaling_policy(
            cloudwatch.Alarms.high_cpu(threshold=high), Fn.Ref(asg), downscale=False,
        ).with_prefixed_names("ScaleUp")
    ).merge_stack(
        simple_scaling_policy(
            cloudwatch.Alarms.low_cpu(threshold=low), Fn.Ref(asg), downscale=True,
        ).with_prefixed_names("ScaleDown")
    )

    return stack


def simple_scaling_policy(alarm, asg_name, downscale=False):
    """Create a simple scaling policy using the supplied alarm."""
    stack = Stack(Description="Resources for a single scaling policy.")

    scaling_policy = ScalingPolicy(
        Properties=ScalingPolicyProperties(
            AdjustmentType="ChangeInCapacity",  # TODO consider making this a lookup value
            AutoScalingGroupName=asg_name,
            Cooldown=1,
            ScalingAdjustment=-1 if downscale else 1,
        ),
    )
    stack.Resources["ScalingPolicy"] = scaling_policy

    # TODO need properties to auto-create empty lists.
    if alarm.Properties.AlarmActions is None:
        alarm.Properties.AlarmActions = []
    if alarm.Properties.Dimensions is None:
        alarm.Properties.Dimensions = []

    alarm.Properties.AlarmActions.append(Fn.Ref(scaling_policy))
    alarm.Properties.Dimensions.append(
        # TODO logical class that wraps this up instead, and allows you to express in a mroe convenient way
        dict(
            Name="AutoScalingGroupName",
            Value=asg_name,
        )
    )
    stack.Resources["ScalingAlarm"] = alarm

    return stack



"""Raw representations of every data type in the AWS ApiGatewayV2 service.

See Also:
    `AWS developer guide for ApiGatewayV2
    <https://docs.aws.amazon.com/apigateway/latest/developerguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Api",
    "ApiProperties",
    "Authorizer",
    "AuthorizerProperties",
    "Deployment",
    "DeploymentProperties",
    "Integration",
    "IntegrationProperties",
    "IntegrationResponse",
    "IntegrationResponseProperties",
    "Model",
    "ModelProperties",
    "Route",
    "RouteProperties",
    "RouteResponse",
    "RouteResponseProperties",
    "Stage",
    "StageProperties",
]


@attrs(**ATTRSCONFIG)
class ApiProperties(ResourceProperties):
    ApiKeySelectionExpression = attrib(default=None)
    Description = attrib(default=None)
    DisableSchemaValidation = attrib(default=None)
    Name = attrib(default=None)
    ProtocolType = attrib(default=None)
    RouteSelectionExpression = attrib(default=None)
    Version = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Api(Resource):
    """A Api for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Api
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-api.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Api"

    Properties: ApiProperties = attrib(
        factory=ApiProperties,
        converter=create_object_converter(ApiProperties),
    )


@attrs(**ATTRSCONFIG)
class AuthorizerProperties(ResourceProperties):
    ApiId = attrib(default=None)
    AuthorizerCredentialsArn = attrib(default=None)
    AuthorizerResultTtlInSeconds = attrib(default=None)
    AuthorizerType = attrib(default=None)
    AuthorizerUri = attrib(default=None)
    IdentitySource = attrib(default=None)
    IdentityValidationExpression = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Authorizer(Resource):
    """A Authorizer for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Authorizer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-authorizer.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Authorizer"

    Properties: AuthorizerProperties = attrib(
        factory=AuthorizerProperties,
        converter=create_object_converter(AuthorizerProperties),
    )


@attrs(**ATTRSCONFIG)
class DeploymentProperties(ResourceProperties):
    ApiId = attrib(default=None)
    Description = attrib(default=None)
    StageName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Deployment(Resource):
    """A Deployment for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Deployment
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-deployment.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Deployment"

    Properties: DeploymentProperties = attrib(
        factory=DeploymentProperties,
        converter=create_object_converter(DeploymentProperties),
    )


@attrs(**ATTRSCONFIG)
class IntegrationProperties(ResourceProperties):
    ApiId = attrib(default=None)
    ConnectionType = attrib(default=None)
    ContentHandlingStrategy = attrib(default=None)
    CredentialsArn = attrib(default=None)
    Description = attrib(default=None)
    IntegrationMethod = attrib(default=None)
    IntegrationType = attrib(default=None)
    IntegrationUri = attrib(default=None)
    PassthroughBehavior = attrib(default=None)
    RequestParameters = attrib(default=None)
    RequestTemplates = attrib(default=None)
    TemplateSelectionExpression = attrib(default=None)
    TimeoutInMillis = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Integration(Resource):
    """A Integration for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Integration
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-integration.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Integration"

    Properties: IntegrationProperties = attrib(
        factory=IntegrationProperties,
        converter=create_object_converter(IntegrationProperties),
    )


@attrs(**ATTRSCONFIG)
class IntegrationResponseProperties(ResourceProperties):
    ApiId = attrib(default=None)
    ContentHandlingStrategy = attrib(default=None)
    IntegrationId = attrib(default=None)
    IntegrationResponseKey = attrib(default=None)
    ResponseParameters = attrib(default=None)
    ResponseTemplates = attrib(default=None)
    TemplateSelectionExpression = attrib(default=None)


@attrs(**ATTRSCONFIG)
class IntegrationResponse(Resource):
    """A Integration Response for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for IntegrationResponse
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-integrationresponse.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::IntegrationResponse"

    Properties: IntegrationResponseProperties = attrib(
        factory=IntegrationResponseProperties,
        converter=create_object_converter(IntegrationResponseProperties),
    )


@attrs(**ATTRSCONFIG)
class ModelProperties(ResourceProperties):
    ApiId = attrib(default=None)
    ContentType = attrib(default=None)
    Description = attrib(default=None)
    Name = attrib(default=None)
    Schema = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Model(Resource):
    """A Model for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Model
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-model.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Model"

    Properties: ModelProperties = attrib(
        factory=ModelProperties,
        converter=create_object_converter(ModelProperties),
    )


@attrs(**ATTRSCONFIG)
class RouteProperties(ResourceProperties):
    ApiId = attrib(default=None)
    ApiKeyRequired = attrib(default=None)
    AuthorizationScopes = attrib(default=None)
    AuthorizationType = attrib(default=None)
    AuthorizerId = attrib(default=None)
    ModelSelectionExpression = attrib(default=None)
    OperationName = attrib(default=None)
    RequestModels = attrib(default=None)
    RequestParameters = attrib(default=None)
    RouteKey = attrib(default=None)
    RouteResponseSelectionExpression = attrib(default=None)
    Target = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Route(Resource):
    """A Route for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Route
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-route.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Route"

    Properties: RouteProperties = attrib(
        factory=RouteProperties,
        converter=create_object_converter(RouteProperties),
    )


@attrs(**ATTRSCONFIG)
class RouteResponseProperties(ResourceProperties):
    ApiId = attrib(default=None)
    ModelSelectionExpression = attrib(default=None)
    ResponseModels = attrib(default=None)
    ResponseParameters = attrib(default=None)
    RouteId = attrib(default=None)
    RouteResponseKey = attrib(default=None)


@attrs(**ATTRSCONFIG)
class RouteResponse(Resource):
    """A Route Response for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for RouteResponse
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-routeresponse.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::RouteResponse"

    Properties: RouteResponseProperties = attrib(
        factory=RouteResponseProperties,
        converter=create_object_converter(RouteResponseProperties),
    )


@attrs(**ATTRSCONFIG)
class StageProperties(ResourceProperties):
    AccessLogSettings = attrib(default=None)
    ApiId = attrib(default=None)
    ClientCertificateId = attrib(default=None)
    DefaultRouteSettings = attrib(default=None)
    DeploymentId = attrib(default=None)
    Description = attrib(default=None)
    RouteSettings = attrib(default=None)
    StageName = attrib(default=None)
    StageVariables = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stage(Resource):
    """A Stage for ApiGatewayV2.

    See Also:
        `AWS Cloud Formation documentation for Stage
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigatewayv2-stage.html>`_
    """

    RESOURCE_TYPE = "AWS::ApiGatewayV2::Stage"

    Properties: StageProperties = attrib(
        factory=StageProperties,
        converter=create_object_converter(StageProperties),
    )



"""General-use classes to interact with the RoboMaker service through CloudFormation.

See Also:
    `AWS developer guide for RoboMaker
    <https://docs.aws.amazon.com/robomaker/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import robomaker as _raw

# noinspection PyUnresolvedReferences
from .._raw.robomaker import *



"""Raw representations of every data type in the AWS CodeDeploy service.

See Also:
    `AWS developer guide for CodeDeploy
    <https://docs.aws.amazon.com/codedeploy/latest/userguide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Application",
    "ApplicationProperties",
    "DeploymentConfig",
    "DeploymentConfigProperties",
    "DeploymentGroup",
    "DeploymentGroupProperties",
]


@attrs(**ATTRSCONFIG)
class ApplicationProperties(ResourceProperties):
    ApplicationName = attrib(default=None)
    ComputePlatform = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Application(Resource):
    """A Application for CodeDeploy.

    See Also:
        `AWS Cloud Formation documentation for Application
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-application.html>`_
    """

    RESOURCE_TYPE = "AWS::CodeDeploy::Application"

    Properties: ApplicationProperties = attrib(
        factory=ApplicationProperties,
        converter=create_object_converter(ApplicationProperties),
    )


@attrs(**ATTRSCONFIG)
class DeploymentConfigProperties(ResourceProperties):
    DeploymentConfigName = attrib(default=None)
    MinimumHealthyHosts = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DeploymentConfig(Resource):
    """A Deployment Config for CodeDeploy.

    See Also:
        `AWS Cloud Formation documentation for DeploymentConfig
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentconfig.html>`_
    """

    RESOURCE_TYPE = "AWS::CodeDeploy::DeploymentConfig"

    Properties: DeploymentConfigProperties = attrib(
        factory=DeploymentConfigProperties,
        converter=create_object_converter(DeploymentConfigProperties),
    )


@attrs(**ATTRSCONFIG)
class DeploymentGroupProperties(ResourceProperties):
    AlarmConfiguration = attrib(default=None)
    ApplicationName = attrib(default=None)
    AutoRollbackConfiguration = attrib(default=None)
    AutoScalingGroups = attrib(default=None)
    Deployment = attrib(default=None)
    DeploymentConfigName = attrib(default=None)
    DeploymentGroupName = attrib(default=None)
    DeploymentStyle = attrib(default=None)
    Ec2TagFilters = attrib(default=None)
    Ec2TagSet = attrib(default=None)
    LoadBalancerInfo = attrib(default=None)
    OnPremisesInstanceTagFilters = attrib(default=None)
    OnPremisesTagSet = attrib(default=None)
    ServiceRoleArn = attrib(default=None)
    TriggerConfigurations = attrib(default=None)


@attrs(**ATTRSCONFIG)
class DeploymentGroup(Resource):
    """A Deployment Group for CodeDeploy.

    See Also:
        `AWS Cloud Formation documentation for DeploymentGroup
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html>`_
    """

    RESOURCE_TYPE = "AWS::CodeDeploy::DeploymentGroup"

    Properties: DeploymentGroupProperties = attrib(
        factory=DeploymentGroupProperties,
        converter=create_object_converter(DeploymentGroupProperties),
    )



"""General-use classes to interact with the SNS service through CloudFormation.

See Also:
    `AWS developer guide for SNS
    <https://docs.aws.amazon.com/sns/latest/dg/welcome.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import sns as _raw

# noinspection PyUnresolvedReferences
from .._raw.sns import *



"""Raw representations of every data type in the AWS Kinesis service.

See Also:
    `AWS developer guide for Kinesis
    <https://docs.aws.amazon.com/guardduty/latest/ug/what-is-guardduty.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Stream",
    "StreamProperties",
    "StreamConsumer",
    "StreamConsumerProperties",
]


@attrs(**ATTRSCONFIG)
class StreamProperties(ResourceProperties):
    Name = attrib(default=None)
    RetentionPeriodHours = attrib(default=None)
    ShardCount = attrib(default=None)
    StreamEncryption = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stream(Resource):
    """A Stream for Kinesis.

    See Also:
        `AWS Cloud Formation documentation for Stream
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html>`_
    """

    RESOURCE_TYPE = "AWS::Kinesis::Stream"

    Properties: StreamProperties = attrib(
        factory=StreamProperties,
        converter=create_object_converter(StreamProperties),
    )


@attrs(**ATTRSCONFIG)
class StreamConsumerProperties(ResourceProperties):
    ConsumerName = attrib(default=None)
    StreamARN = attrib(default=None)


@attrs(**ATTRSCONFIG)
class StreamConsumer(Resource):
    """A Stream Consumer for Kinesis.

    See Also:
        `AWS Cloud Formation documentation for StreamConsumer
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-streamconsumer.html>`_
    """

    RESOURCE_TYPE = "AWS::Kinesis::StreamConsumer"

    Properties: StreamConsumerProperties = attrib(
        factory=StreamConsumerProperties,
        converter=create_object_converter(StreamConsumerProperties),
    )



import os

from setuptools import find_packages
from setuptools import setup

THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))


def get_version_data():
    data = {}
    with open(os.path.join(THIS_DIRECTORY, "src", "flyingcircus", "_about.py")) as fp:
        exec(fp.read(), data)
    return data


def get_readme():
    with open(os.path.join(THIS_DIRECTORY, "README.md")) as fp:
        desc = fp.read()
    return desc


setup(
    # Basic Project Description
    name="flying-circus",
    version=get_version_data()['__version__'],
    description="A tool for describing AWS infrastructure.",
    long_description=get_readme(),
    long_description_content_type="text/markdown",

    # Project Packaging
    package_dir={'': 'src'},
    packages=find_packages(where="src"),  # FIXME do we need this?
    install_requires=[
        # We use the `kw_only` only for attribute classes, which was
        # introduced in v18.2.0
        'attrs>=18.2.0',
        # TODO #165 PyYAML v3 has vulnerability that does not affect us, but we should upgrade as soon as a fix is available
        'PyYAML==3.13',
    ],

    # Contact Details
    author="Gary Donovan",
    author_email="gazza@gazza.id.au",
    url="https://github.com/garyd203/flying-circus",
    project_urls={
        'Documentation': "https://flying-circus.readthedocs.io/en/latest/",
        'Source': "https://github.com/garyd203/flying-circus",
        'Tracker': "https://github.com/garyd203/flying-circus/issues",
    },

    # Other metadata for PyPI
    license="LGPL v3",
    keywords="AWS cloudformation infrastructure-as-code",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Pre-processors',
        'Topic :: System :: Systems Administration',
    ],
    metadata_version="2.1",
)



"""General-use classes to interact with the CodeCommit service through CloudFormation.

See Also:
    `AWS developer guide for CodeCommit
    <https://docs.aws.amazon.com/codecommit/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import codecommit as _raw

# noinspection PyUnresolvedReferences
from .._raw.codecommit import *



"""Raw representations of every data type in the AWS IAM service.

See Also:
    `AWS developer guide for IAM
    <https://docs.aws.amazon.com/IAM/latest/UserGuide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "AccessKey",
    "AccessKeyProperties",
    "Group",
    "GroupProperties",
    "InstanceProfile",
    "InstanceProfileProperties",
    "ManagedPolicy",
    "ManagedPolicyProperties",
    "Policy",
    "PolicyProperties",
    "Role",
    "RoleProperties",
    "ServiceLinkedRole",
    "ServiceLinkedRoleProperties",
    "User",
    "UserProperties",
    "UserToGroupAddition",
    "UserToGroupAdditionProperties",
]


@attrs(**ATTRSCONFIG)
class AccessKeyProperties(ResourceProperties):
    Serial = attrib(default=None)
    Status = attrib(default=None)
    UserName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class AccessKey(Resource):
    """A Access Key for IAM.

    See Also:
        `AWS Cloud Formation documentation for AccessKey
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::AccessKey"

    Properties: AccessKeyProperties = attrib(
        factory=AccessKeyProperties,
        converter=create_object_converter(AccessKeyProperties),
    )


@attrs(**ATTRSCONFIG)
class GroupProperties(ResourceProperties):
    GroupName = attrib(default=None)
    ManagedPolicyArns = attrib(default=None)
    Path = attrib(default=None)
    Policies = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Group(Resource):
    """A Group for IAM.

    See Also:
        `AWS Cloud Formation documentation for Group
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::Group"

    Properties: GroupProperties = attrib(
        factory=GroupProperties,
        converter=create_object_converter(GroupProperties),
    )


@attrs(**ATTRSCONFIG)
class InstanceProfileProperties(ResourceProperties):
    InstanceProfileName = attrib(default=None)
    Path = attrib(default=None)
    Roles = attrib(default=None)


@attrs(**ATTRSCONFIG)
class InstanceProfile(Resource):
    """A Instance Profile for IAM.

    See Also:
        `AWS Cloud Formation documentation for InstanceProfile
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::InstanceProfile"

    Properties: InstanceProfileProperties = attrib(
        factory=InstanceProfileProperties,
        converter=create_object_converter(InstanceProfileProperties),
    )


@attrs(**ATTRSCONFIG)
class ManagedPolicyProperties(ResourceProperties):
    Description = attrib(default=None)
    Groups = attrib(default=None)
    ManagedPolicyName = attrib(default=None)
    Path = attrib(default=None)
    PolicyDocument = attrib(default=None)
    Roles = attrib(default=None)
    Users = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ManagedPolicy(Resource):
    """A Managed Policy for IAM.

    See Also:
        `AWS Cloud Formation documentation for ManagedPolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::ManagedPolicy"

    Properties: ManagedPolicyProperties = attrib(
        factory=ManagedPolicyProperties,
        converter=create_object_converter(ManagedPolicyProperties),
    )


@attrs(**ATTRSCONFIG)
class PolicyProperties(ResourceProperties):
    Groups = attrib(default=None)
    PolicyDocument = attrib(default=None)
    PolicyName = attrib(default=None)
    Roles = attrib(default=None)
    Users = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Policy(Resource):
    """A Policy for IAM.

    See Also:
        `AWS Cloud Formation documentation for Policy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::Policy"

    Properties: PolicyProperties = attrib(
        factory=PolicyProperties,
        converter=create_object_converter(PolicyProperties),
    )


@attrs(**ATTRSCONFIG)
class RoleProperties(ResourceProperties):
    AssumeRolePolicyDocument = attrib(default=None)
    ManagedPolicyArns = attrib(default=None)
    MaxSessionDuration = attrib(default=None)
    Path = attrib(default=None)
    PermissionsBoundary = attrib(default=None)
    Policies = attrib(default=None)
    RoleName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Role(Resource):
    """A Role for IAM.

    See Also:
        `AWS Cloud Formation documentation for Role
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::Role"

    Properties: RoleProperties = attrib(
        factory=RoleProperties,
        converter=create_object_converter(RoleProperties),
    )


@attrs(**ATTRSCONFIG)
class ServiceLinkedRoleProperties(ResourceProperties):
    AWSServiceName = attrib(default=None)
    CustomSuffix = attrib(default=None)
    Description = attrib(default=None)


@attrs(**ATTRSCONFIG)
class ServiceLinkedRole(Resource):
    """A Service Linked Role for IAM.

    See Also:
        `AWS Cloud Formation documentation for ServiceLinkedRole
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-servicelinkedrole.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::ServiceLinkedRole"

    Properties: ServiceLinkedRoleProperties = attrib(
        factory=ServiceLinkedRoleProperties,
        converter=create_object_converter(ServiceLinkedRoleProperties),
    )


@attrs(**ATTRSCONFIG)
class UserProperties(ResourceProperties):
    Groups = attrib(default=None)
    LoginProfile = attrib(default=None)
    ManagedPolicyArns = attrib(default=None)
    Path = attrib(default=None)
    PermissionsBoundary = attrib(default=None)
    Policies = attrib(default=None)
    UserName = attrib(default=None)


@attrs(**ATTRSCONFIG)
class User(Resource):
    """A User for IAM.

    See Also:
        `AWS Cloud Formation documentation for User
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::User"

    Properties: UserProperties = attrib(
        factory=UserProperties,
        converter=create_object_converter(UserProperties),
    )


@attrs(**ATTRSCONFIG)
class UserToGroupAdditionProperties(ResourceProperties):
    GroupName = attrib(default=None)
    Users = attrib(default=None)


@attrs(**ATTRSCONFIG)
class UserToGroupAddition(Resource):
    """A User To Group Addition for IAM.

    See Also:
        `AWS Cloud Formation documentation for UserToGroupAddition
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html>`_
    """

    RESOURCE_TYPE = "AWS::IAM::UserToGroupAddition"

    Properties: UserToGroupAdditionProperties = attrib(
        factory=UserToGroupAdditionProperties,
        converter=create_object_converter(UserToGroupAdditionProperties),
    )



"""General-use classes to interact with the Events service through CloudFormation.

See Also:
    `AWS developer guide for Events
    <https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/WhatIsCloudWatchEvents.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import events as _raw

# noinspection PyUnresolvedReferences
from .._raw.events import *



"""Raw representations of every data type in the AWS CloudFormation service.

See Also:
    `AWS developer guide for CloudFormation
    <https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/index.html>`_

This file is automatically generated, and should not be directly edited.
"""

from typing import Any
from typing import Dict

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "CustomResource",
    "CustomResourceProperties",
    "Macro",
    "MacroProperties",
    "Stack",
    "StackProperties",
    "WaitCondition",
    "WaitConditionProperties",
    "WaitConditionHandle",
    "WaitConditionHandleProperties",
]


@attrs(**ATTRSCONFIG)
class CustomResourceProperties(ResourceProperties):
    ServiceToken = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CustomResource(Resource):
    """A Custom Resource for CloudFormation.

    See Also:
        `AWS Cloud Formation documentation for CustomResource
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cfn-customresource.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFormation::CustomResource"

    Properties: CustomResourceProperties = attrib(
        factory=CustomResourceProperties,
        converter=create_object_converter(CustomResourceProperties),
    )


@attrs(**ATTRSCONFIG)
class MacroProperties(ResourceProperties):
    Description = attrib(default=None)
    FunctionName = attrib(default=None)
    LogGroupName = attrib(default=None)
    LogRoleARN = attrib(default=None)
    Name = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Macro(Resource):
    """A Macro for CloudFormation.

    See Also:
        `AWS Cloud Formation documentation for Macro
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudformation-macro.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFormation::Macro"

    Properties: MacroProperties = attrib(
        factory=MacroProperties,
        converter=create_object_converter(MacroProperties),
    )


@attrs(**ATTRSCONFIG)
class StackProperties(ResourceProperties):
    NotificationARNs = attrib(default=None)
    Parameters = attrib(default=None)
    Tags = attrib(default=None)
    TemplateURL = attrib(default=None)
    TimeoutInMinutes = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Stack(Resource):
    """A Stack for CloudFormation.

    See Also:
        `AWS Cloud Formation documentation for Stack
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFormation::Stack"

    Properties: StackProperties = attrib(
        factory=StackProperties,
        converter=create_object_converter(StackProperties),
    )


@attrs(**ATTRSCONFIG)
class WaitConditionProperties(ResourceProperties):
    Count = attrib(default=None)
    Handle = attrib(default=None)
    Timeout = attrib(default=None)


@attrs(**ATTRSCONFIG)
class WaitCondition(Resource):
    """A Wait Condition for CloudFormation.

    See Also:
        `AWS Cloud Formation documentation for WaitCondition
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFormation::WaitCondition"

    Properties: WaitConditionProperties = attrib(
        factory=WaitConditionProperties,
        converter=create_object_converter(WaitConditionProperties),
    )

    # NB: CreationPolicy may be set for WaitCondition
    # (unlike most Resource types)
    CreationPolicy: Dict[str, Any] = attrib(factory=dict)


@attrs(**ATTRSCONFIG)
class WaitConditionHandleProperties(ResourceProperties):
    # A WaitConditionHandle doesn't actually have any Properties.
    pass


@attrs(**ATTRSCONFIG)
class WaitConditionHandle(Resource):
    """A Wait Condition Handle for CloudFormation.

    See Also:
        `AWS Cloud Formation documentation for WaitConditionHandle
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitconditionhandle.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFormation::WaitConditionHandle"

    Properties: WaitConditionHandleProperties = attrib(
        factory=WaitConditionHandleProperties,
        converter=create_object_converter(WaitConditionHandleProperties),
    )



"""General-use classes to interact with the ElasticLoadBalancingV2 service through CloudFormation.

See Also:
    `AWS developer guide for ElasticLoadBalancingV2
    <https://docs.aws.amazon.com/elasticloadbalancing/latest/application/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import elasticloadbalancingv2 as _raw

# noinspection PyUnresolvedReferences
from .._raw.elasticloadbalancingv2 import *



"""General-use classes to interact with the ServiceDiscovery service through CloudFormation.

See Also:
    `AWS developer guide for ServiceDiscovery
    <https://docs.aws.amazon.com/cloud-map/latest/dg/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import servicediscovery as _raw

# noinspection PyUnresolvedReferences
from .._raw.servicediscovery import *



"""General-use classes to interact with the OpsWorks service through CloudFormation.

See Also:
    `AWS developer guide for OpsWorks
    <https://docs.aws.amazon.com/opsworks/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import opsworks as _raw

# noinspection PyUnresolvedReferences
from .._raw.opsworks import *



"""General-use classes to interact with the KinesisAnalytics service through CloudFormation.

See Also:
    `AWS developer guide for KinesisAnalytics
    <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/what-is.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import kinesisanalytics as _raw

# noinspection PyUnresolvedReferences
from .._raw.kinesisanalytics import *



"""General-use classes to interact with the Neptune service through CloudFormation.

See Also:
    `AWS developer guide for Neptune
    <https://docs.aws.amazon.com/neptune/latest/userguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import neptune as _raw

# noinspection PyUnresolvedReferences
from .._raw.neptune import *



"""General-use classes to interact with the AutoScalingPlans service through CloudFormation.

See Also:
    `AWS developer guide for AutoScalingPlans
    <https://docs.aws.amazon.com/autoscaling/plans/userguide/>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import autoscalingplans as _raw

# noinspection PyUnresolvedReferences
from .._raw.autoscalingplans import *



"""Raw representations of every data type in the AWS CloudFront service.

See Also:
    `AWS developer guide for CloudFront
    <https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Introduction.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "CloudFrontOriginAccessIdentity",
    "CloudFrontOriginAccessIdentityProperties",
    "Distribution",
    "DistributionProperties",
    "StreamingDistribution",
    "StreamingDistributionProperties",
]


@attrs(**ATTRSCONFIG)
class CloudFrontOriginAccessIdentityProperties(ResourceProperties):
    CloudFrontOriginAccessIdentityConfig = attrib(default=None)


@attrs(**ATTRSCONFIG)
class CloudFrontOriginAccessIdentity(Resource):
    """A Cloud Front Origin Access Identity for CloudFront.

    See Also:
        `AWS Cloud Formation documentation for CloudFrontOriginAccessIdentity
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudfront-cloudfrontoriginaccessidentity.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFront::CloudFrontOriginAccessIdentity"

    Properties: CloudFrontOriginAccessIdentityProperties = attrib(
        factory=CloudFrontOriginAccessIdentityProperties,
        converter=create_object_converter(CloudFrontOriginAccessIdentityProperties),
    )


@attrs(**ATTRSCONFIG)
class DistributionProperties(ResourceProperties):
    DistributionConfig = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Distribution(Resource):
    """A Distribution for CloudFront.

    See Also:
        `AWS Cloud Formation documentation for Distribution
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudfront-distribution.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFront::Distribution"

    Properties: DistributionProperties = attrib(
        factory=DistributionProperties,
        converter=create_object_converter(DistributionProperties),
    )


@attrs(**ATTRSCONFIG)
class StreamingDistributionProperties(ResourceProperties):
    StreamingDistributionConfig = attrib(default=None)
    Tags = attrib(default=None)


@attrs(**ATTRSCONFIG)
class StreamingDistribution(Resource):
    """A Streaming Distribution for CloudFront.

    See Also:
        `AWS Cloud Formation documentation for StreamingDistribution
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudfront-streamingdistribution.html>`_
    """

    RESOURCE_TYPE = "AWS::CloudFront::StreamingDistribution"

    Properties: StreamingDistributionProperties = attrib(
        factory=StreamingDistributionProperties,
        converter=create_object_converter(StreamingDistributionProperties),
    )



"""General-use classes to interact with the ApiGatewayV2 service through CloudFormation.

See Also:
    `AWS developer guide for ApiGatewayV2
    <https://docs.aws.amazon.com/apigateway/latest/developerguide/index.html>`_
"""

# noinspection PyUnresolvedReferences
from .._raw import apigatewayv2 as _raw

# noinspection PyUnresolvedReferences
from .._raw.apigatewayv2 import *



"""General-use classes to interact with the SSM service through CloudFormation.

See Also:
    `AWS developer guide for SSM
    <https://docs.aws.amazon.com/systems-manager/latest/APIReference/Welcome.html>`_
"""

from .._raw import ssm as _raw

# noinspection PyUnresolvedReferences
from .._raw.ssm import *

# Also provide an alias for the SSM Parameter resource, to reduce conflicts
# with the top-level template Parameter object
SSMParameter = _raw.Parameter
SSMParameterProperties = _raw.ParameterProperties



"""Raw representations of every data type in the AWS S3 service.

See Also:
    `AWS developer guide for S3
    <https://docs.aws.amazon.com/AmazonS3/latest/dev/Welcome.html>`_

This file is automatically generated, and should not be directly edited.
"""

from attr import attrib
from attr import attrs

from ..core import ATTRSCONFIG
from ..core import Resource
from ..core import ResourceProperties
from ..core import create_object_converter

__all__ = [
    "Bucket",
    "BucketProperties",
    "BucketPolicy",
    "BucketPolicyProperties",
]


@attrs(**ATTRSCONFIG)
class BucketProperties(ResourceProperties):
    AccelerateConfiguration = attrib(default=None)
    AccessControl = attrib(default=None)
    AnalyticsConfigurations = attrib(default=None)
    BucketEncryption = attrib(default=None)
    BucketName = attrib(default=None)
    CorsConfiguration = attrib(default=None)
    InventoryConfigurations = attrib(default=None)
    LifecycleConfiguration = attrib(default=None)
    LoggingConfiguration = attrib(default=None)
    MetricsConfigurations = attrib(default=None)
    NotificationConfiguration = attrib(default=None)
    PublicAccessBlockConfiguration = attrib(default=None)
    ReplicationConfiguration = attrib(default=None)
    Tags = attrib(default=None)
    VersioningConfiguration = attrib(default=None)
    WebsiteConfiguration = attrib(default=None)


@attrs(**ATTRSCONFIG)
class Bucket(Resource):
    """A Bucket for S3.

    See Also:
        `AWS Cloud Formation documentation for Bucket
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html>`_
    """

    RESOURCE_TYPE = "AWS::S3::Bucket"

    Properties: BucketProperties = attrib(
        factory=BucketProperties,
        converter=create_object_converter(BucketProperties),
    )


@attrs(**ATTRSCONFIG)
class BucketPolicyProperties(ResourceProperties):
    Bucket = attrib(default=None)
    PolicyDocument = attrib(default=None)


@attrs(**ATTRSCONFIG)
class BucketPolicy(Resource):
    """A Bucket Policy for S3.

    See Also:
        `AWS Cloud Formation documentation for BucketPolicy
        <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html>`_
    """

    RESOURCE_TYPE = "AWS::S3::BucketPolicy"

    Properties: BucketPolicyProperties = attrib(
        factory=BucketPolicyProperties,
        converter=create_object_converter(BucketPolicyProperties),
    )
